﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using FarPoint.Win.Spread;
using FarPoint.Win.Spread.Model;
using FarPoint.Win.Spread.CellType;
using GrapeCity.Spreadsheet;

namespace SpreadWinDemo.CellValidator
{
  public partial class CellValidator : Form
  {
    public CellValidator()
    {
      InitializeComponent();

      setData();
   

    }
    #region Setting for fpSpread1
    private void button1_Click(object sender, EventArgs e)
    {
      fpSpread1.ShowCellValidatorsForm(new Form(), true);

    }

    private void button2_Click(object sender, EventArgs e)
    {
      CellRange range = fpSpread1.ActiveSheet.GetSelection(0);
      fpSpread1.ActiveSheet.ClearValidators(range.Row, range.Column, range.RowCount, range.ColumnCount);
    }
    #endregion

    #region Setting for fpSpread2
    public void setData()
    {
      TextCellType tx = new TextCellType();
      tx.Multiline = true;
      fpSpread2.Sheets.Count = 2;
      fpSpread2.Sheets[0].SheetName = "Names";
      fpSpread2.Sheets[1].SheetName = "Schedules";
      fpSpread2.Sheets[0].AddCustomName("SchedNames", "Names!$C$2:$C$100", 0, 0, false, "");
      fpSpread2.Sheets[0].AddCustomName("SchedDate", "Schedules!$F$4", 0, 0, false, "");
      fpSpread2.AllowUserFormulas = true;
      for (int i = 0; i < 2; i++)
      {
        fpSpread2.Sheets[i].ColumnHeader.Visible = false;
        fpSpread2.Sheets[i].RowHeader.Visible = false;
        fpSpread2.Sheets[i].VerticalGridLine = new GridLine(GridLineType.None);
        fpSpread2.Sheets[i].HorizontalGridLine = new GridLine(GridLineType.None);

      }
      // Setting for validator
      //Range Validator alow enter datetime from 1/1/2017 to 12/31/2017
      RangeValidator rangeValidator = new RangeValidator();
      rangeValidator.MinValue = new DateTime(2017, 1, 1);
      rangeValidator.MaxValue = new DateTime(2017, 12, 31);
      rangeValidator.RequiredType = typeof(DateTime);

      MessageBoxNotify messRangeNotify = new MessageBoxNotify();
      messRangeNotify.Message = "Pls input date from 1/1/2017 to 12/31/2017";
      messRangeNotify.DoActionReason = ValidateReasons.EndEdit;

      LineNotify lineNotify = new LineNotify();
      lineNotify.DoActionReason = ValidateReasons.EndEdit;
      rangeValidator.Actions.Add(messRangeNotify);
      rangeValidator.Actions.Add(lineNotify);

      // Require type Validator, only allow enter datetime value
      RequiredTypeValidator requireTypeValidator = new RequiredTypeValidator();
      requireTypeValidator.RequiredType = typeof(System.DateTime);
      MessageBoxNotify messRequireNotify = new MessageBoxNotify();
      messRequireNotify.Message = "Pls input datetime value";
      messRequireNotify.DoActionReason = ValidateReasons.EndEdit;

      requireTypeValidator.Actions.Add(messRequireNotify);
      requireTypeValidator.Actions.Add(lineNotify);
      rangeValidator.Actions.Add(lineNotify);
      // TextLength validator, only allow enter 2 to 15 character in name
      TextLengthValidator textlengthValidator = new TextLengthValidator();
      textlengthValidator.MinimumLength = 2;
      textlengthValidator.MaximumLength = 15;

      MessageBoxNotify messTextNotify = new MessageBoxNotify();
      messTextNotify.Message = "Pls enter 2 to 15 characters in name";
      messTextNotify.DoActionReason = ValidateReasons.EndEdit;

      textlengthValidator.Actions.Add(messTextNotify);
      textlengthValidator.Actions.Add(lineNotify);
     

      #region Sheet2

      fpSpread2.Sheets[0].SetClip(0, 0, 1, 3, "Employee\tName & Date\tShowName");
      fpSpread2.Sheets[0].Cells["A1:C1"].Font = new System.Drawing.Font("Times New Roman", 10, System.Drawing.FontStyle.Bold);
      fpSpread2.Sheets[0].Cells["A1:C1"].BackColor = System.Drawing.Color.FromArgb(235, 241, 222);
      fpSpread2.Sheets[0].SetClip(1, 0, 11, 1, " Patricia Gabel \nPail Molden \nLyle Kier \nChandra Ordidge \nPennie Murdie \nKelci Kopelman \nTeirtza Infantino \nJamil Vasilkov \nTine Thickens \n  Dorisa Tompion \n  Belle McGlaud");
      fpSpread2.Sheets[0].Cells["A2"].Text = "Anna";
      fpSpread2.Sheets[0].Cells[1, 1, 99, 1].Formula = @"IF(A2<>"""",A2&SchedDate,"""")";
      fpSpread2.Sheets[0].Cells[1, 2, 99, 2].Formula = @"IF(COUNTIF(Schedules!C:C,B2)>0,"""",Names!A2)";
      fpSpread2.Sheets[0].Cells[0, 0, 99, 2].Border = new FarPoint.Win.LineBorder(System.Drawing.Color.Black);
      fpSpread2.Sheets[0].Columns[0, 2].Width = 150;
      fpSpread2.Sheets[0].AddValidators(new CellRange(1, 0, 99, 1), textlengthValidator);
      fpSpread2.Sheets[0].Cells[0, 0].Note = "Employee Names:\nTyped in list of employee names (you could use full names instead)";
      fpSpread2.Sheets[0].Cells[0, 1].Note = "Name / Date Check:\nThis formula combines the employee name  and the work date (from the named range - SchedDate - on the Schedule sheet).\n=A15 & SchedDate";
      fpSpread2.Sheets[0].Cells[0, 2].Note = @"Names for Schedule Sheet:\nThese names are used for the Data Validation list in column B of Schedule sheet. The cells have been named: SchedNames\nIf the name has been assigned for the current date it is blanked out of this list\nIf the name has NOT been assigned for the current date the employee name from Column A is shown.\n=IF(COUNTIF(Schedule!C:C,B2)>0,"",'Setup 2'!A2)";
      fpSpread2.Sheets[0].RowCount = 100;
      fpSpread2.Sheets[0].ColumnCount = 6;
      fpSpread2.Sheets[0].FrozenRowCount = 1;
      #endregion

      #region Sheet3
      fpSpread2.Sheets[1].SetClip(0, 0, 1, 3, " Work Date\tEmployee\tName & Date");
      fpSpread2.Sheets[1].Cells["A1:C1"].Font= new System.Drawing.Font("Times New Roman", 10, System.Drawing.FontStyle.Bold);

      fpSpread2.Sheets[1].Cells["A1:C1"].BackColor = System.Drawing.Color.FromArgb(235, 241, 222);
      for (int i = 1; i < 10; i++)
      {
        fpSpread2.Sheets[1].Cells[i, 0].Text = new DateTime(2017, 1, i).ToShortDateString();
      }
      fpSpread2.Sheets[1].Cells[1, 2, 99, 2].Formula = @"IF(B2<>"""",B2&A2,"""")";
      fpSpread2.Sheets[1].Cells["F4"].BackColor = System.Drawing.Color.LightYellow;
      fpSpread2.Sheets[1].Cells["F4"].Text = new DateTime(2017, 1, 1).ToShortDateString();
      fpSpread2.Sheets[1].Cells[0, 0, 99, 2].Border = new FarPoint.Win.LineBorder(System.Drawing.Color.Black);
      fpSpread2.Sheets[1].Columns[0, 2].Width = 150;
      fpSpread2.Sheets[1].AddValidators(new CellRange(1, 0, 99, 1), requireTypeValidator);
      fpSpread2.Sheets[1].AddValidators(new CellRange(3, 5, 1, 1), rangeValidator);
      for (int i = 1; i < 100; i++)
      {
        if (fpSpread2.Sheets[1].Cells[i, 0].Text == fpSpread2.Sheets[1].Cells[3, 5].Text)
        {
          fpSpread2.Sheets[1].Validations.Clear(i, 1, i, 1);
          var validation = fpSpread2.Sheets[1].Cells[i, 1].Validation.Add(DataValidationType.List, DataValidationErrorStyle.Warning, DataValidationOperator.Equal, "=SchedNames", null);
          validation.IgnoreBlank = true;
        }
      }

      fpSpread2.Sheets[1].Cells[0, 0].Note = "Work Dates:\nDates are typed in this column. Enter the date once for each employee that you need. In this example, 12 employees are needed July 4th";
      fpSpread2.Sheets[1].Cells[0, 1].Note = "Data Validation:\nThese cells have Data Validation activated. When a cell is selected, a drop-down arrow appears. The user can select from a list of names.\n";
      fpSpread2.Sheets[1].Cells[0, 2].Note = @"Name and Date Check\nThis formula combines the employee name  and the work date (as a serial number).\n=IF(B4<>"",B4&A4,"")\nIf no name has been selected, this cell looks blank.";
      fpSpread2.Sheets[1].Cells[3, 5].Note = "Scheduling Date Cell\nThis cell has been named (SchedDate) so it is easier to use in formulas on the worksheet. \n<--Enter date you wish to schedule";
      fpSpread2.Sheets[1].Cells[3, 5].NoteStyle = NoteStyle.StickyNote;
      fpSpread2.Sheets[1].RowCount = 100;
      fpSpread2.Sheets[1].ColumnCount = 6;
      fpSpread2.Sheets[1].FrozenRowCount = 1;
      fpSpread2.Sheets[1].CellChanged += DataValidator_CellChanged;
      #endregion




    }

    private void DataValidator_CellChanged(object sender, SheetViewEventArgs e)
    {
      if ((e.Column == 0) || ((e.Column == 5) && (e.Row == 3)))
      {
        for (int i = 1; i < 100; i++)
        {
          if (fpSpread2.Sheets[1].Cells[i, 0].Text == fpSpread2.Sheets[1].Cells[3, 5].Text)
          {
            fpSpread2.Sheets[1].Validations.Clear(i, 1, i, 1);

            var validation = fpSpread2.Sheets[1].Cells[i, 1].Validation.Add(DataValidationType.List, DataValidationErrorStyle.Warning, DataValidationOperator.Equal, "=SchedNames", null);
            validation.IgnoreBlank = true;
          }
        }
      }

    }

    #endregion

   
  }
}
