﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using System.Windows.Forms.VisualStyles;
using System.Runtime.InteropServices;
internal enum Msg
{
  // Fields 
  WM_CREATE = 1,
  WM_LBUTTONDBLCLK = 0x203,
  WM_LBUTTONDOWN = 0x201,
  WM_PARENTNOTIFY = 0x210
}
[StructLayout(LayoutKind.Sequential)]
internal struct PointStruct
{
  public int x;
  public int y;
  public PointStruct(int x, int y)
  {
    this.x = x;
    this.y = y;
  }

  public PointStruct(PointStruct pointStruct)
  {
    this.x = pointStruct.x;
    this.y = pointStruct.y;
  }
}
[StructLayout(LayoutKind.Sequential)]
internal struct RECT
{
  public UInt32 left;
  public UInt32 top;
  public UInt32 right;
  public UInt32 bottom;
  public PointStruct Location
  {
    get { return new PointStruct(Convert.ToInt32(this.left), Convert.ToInt32(this.top)); }
    set
    {
      this.right = (this.right - (this.left - Convert.ToUInt32(value.x)));
      this.bottom = (this.bottom - (this.bottom - Convert.ToUInt32(value.y)));
      this.left = Convert.ToUInt32(value.x);
      this.top = Convert.ToUInt32(value.y);
    }
  }

  public UInt32 Width
  {
    get { return (this.right - this.left); }
    set { this.right = (this.left + value); }
  }

  public UInt32 Height
  {
    get { return (this.bottom - this.top); }
    set { this.bottom = (this.top + value); }
  }

  public override string ToString()
  {
    return string.Concat(new object[] {
			this.left,
			":",
			this.top,
			":",
			this.right,
			":",
			this.bottom
		});
  }
}
[StructLayout(LayoutKind.Sequential)]
public class COMRECT
{
  public int left;
  public int top;
  public int right;
  public int bottom;
  public COMRECT()
  {
  }

  public COMRECT(Rectangle r)
  {
    this.left = r.X;
    this.top = r.Y;
    this.right = r.Right;
    this.bottom = r.Bottom;
  }

  public COMRECT(int left, int top, int right, int bottom)
  {
    this.left = left;
    this.top = top;
    this.right = right;
    this.bottom = bottom;
  }

  public static COMRECT FromXYWH(int x, int y, int width, int height)
  {
    return new COMRECT(x, y, (x + width), (y + height));
  }

  public override string ToString()
  {
    return string.Concat(new object[] {
			"Left = ",
			this.left,
			" Top ",
			this.top,
			" Right = ",
			this.right,
			" Bottom = ",
			this.bottom
		});
  }
}
internal class ListBoxNative : NativeWindow
{
  [DllImport("user32.dll", CharSet = CharSet.Auto)]
  private static extern IntPtr GetWindow(IntPtr hWnd, uint wCmd);
  [DllImport("user32.dll", CharSet = CharSet.Auto)]
  private static extern IntPtr WindowFromPoint(int xPoint, int yPoint);
  [DllImport("user32.dll", CharSet = CharSet.Auto, ExactSpelling = true)]
  private static extern int GetMessagePos();
  [DllImport("user32.dll")]
  private static extern bool GetClientRect(IntPtr hwnd, ref RECT rect);
  [DllImport("user32.dll")]
  private static extern IntPtr ClientToScreen(IntPtr hwnd, ref PointStruct lpPoint);
  [DllImport("user32.dll", CharSet = CharSet.Auto, ExactSpelling = true)]
  private static extern bool InvalidateRect(IntPtr hWnd, COMRECT rect, bool erase);
  [DllImport("user32.dll", CharSet = CharSet.Auto, ExactSpelling = true)]
  private static extern bool UpdateWindow(IntPtr hWnd);
  public ListBoxNative(CheckedListComboBox parent)
  {
    this.parent = parent;
    this.parent.MaxDropDownItems = this.parent.Items.Count;
  }
  private bool ProcessMouseDown(ref Message m)
  {
    int a = GetMessagePos();
    int x = (a & 0xffff);
    int y = (int)(a & 0xffff0000) / 0x10000;
    PointStruct p = new PointStruct();
    p.x = x;
    p.y = y;
    RECT r = new RECT();
    GetClientRect(this.Handle, ref r);
    PointStruct p2 = new PointStruct(0, 0);
    ClientToScreen(this.Handle, ref p2);
    Rectangle rec = new Rectangle(p2.x, p2.y,(int) r.Width,(int) r.Height);
    if ((rec.Contains(new Point(p.x, p.y))))
    {
      int itemIndex = (int) (Math.Floor ( (decimal)( (p.y - rec.Y) / parent.ItemHeight)));
      if ((itemIndex != -1 && parent.Items.Count > 0 && itemIndex < parent.Items.Count))
      {
        CheckItem(itemIndex);
        if (parent.SelectedIndex != itemIndex)
        {
          parent.SelectedIndex = itemIndex;
        }
      }
      m.Result = (IntPtr)1;
      return true;
    }
    return false;
  }
  
  protected override void WndProc(ref Message m)
  {
    switch ((m.Msg))
    {
      case (int)(Msg.WM_LBUTTONDBLCLK):
        if (ProcessMouseDown(ref m))
        {
          return;
        }
        break;
      case (int)(Msg.WM_LBUTTONDOWN):
        if (ProcessMouseDown(ref m))
        {
          return;
        }
        break;
    }
    base.WndProc(ref m);
  }
  private void CheckAll()
  {
    CheckedListComboBoxItem item = null;
    for (int i = 1; i <= parent.Items.Count - 1; i++)
    {
      item = (CheckedListComboBoxItem)parent.Items[i];
      parent.RaiseCheckStateChanged(parent, new ItemCheckEventArgs(i, item.CheckState, CheckState.Checked));
      item.CheckState = CheckState.Checked;
    }
  }

  private void UnCheckAll()
  {
    CheckedListComboBoxItem item = null;
    for (int i = 1; i <= parent.Items.Count - 1; i++)
    {
      item = (CheckedListComboBoxItem)parent.Items[i];
      parent.RaiseCheckStateChanged(parent, new ItemCheckEventArgs(i, CheckState.Unchecked, item.CheckState));
      item.CheckState = CheckState.Unchecked;
    }
  }
  internal void CheckItem(int index)
  {
    CheckedListComboBoxItem item = (CheckedListComboBoxItem)parent.Items[index];
    COMRECT comrec = null;
    CheckState chState = default(CheckState);
    if ((index == 0))
    {
      switch (item.CheckState)
      {
        case CheckState.Checked:
          chState = CheckState.Unchecked;
          UnCheckAll();
          break; 
        case CheckState.Unchecked:
          chState = CheckState.Checked;
          CheckAll();
          break; 
        case CheckState.Indeterminate:
          chState = CheckState.Checked;
          CheckAll();
          break; 
      }
    }
    else
    {
      switch (item.CheckState)
      {
        case CheckState.Checked:
          chState = CheckState.Unchecked;
          bool unCheckedAll = true;
          for (int i = 1; i <= parent.Items.Count - 1; i++)
          {
            if (i == index)
            {
              continue;
            }
            if (((CheckedListComboBoxItem)parent.Items[i]).CheckState == CheckState.Checked)
            {
              unCheckedAll = false;
              break; 
            }
          }

          CheckedListComboBoxItem allItem = (CheckedListComboBoxItem)parent.Items[0];
          comrec = new COMRECT(0, 0, parent.ItemHeight, parent.ItemHeight);
          if (!unCheckedAll)
          {
            parent.RaiseCheckStateChanged(parent, new ItemCheckEventArgs(0, CheckState.Indeterminate, allItem.CheckState));
            allItem.CheckState = CheckState.Indeterminate;
            InvalidateRect(this.Handle, comrec, false);
          }
          else if (allItem.CheckState != CheckState.Unchecked)
          {
            parent.RaiseCheckStateChanged(parent, new ItemCheckEventArgs(0, CheckState.Unchecked, allItem.CheckState));
            allItem.CheckState = CheckState.Unchecked;
            InvalidateRect(this.Handle, comrec, false);
          }
          break; 
        case CheckState.Unchecked:
          chState = CheckState.Checked;
          bool checkedAll = true;
          for (int i = 1; i <= parent.Items.Count - 1; i++)
          {
            if (i == index)
            {
              continue;
            }
            if (((CheckedListComboBoxItem)parent.Items[i]).CheckState == CheckState.Unchecked)
            {
              checkedAll = false;
              break; 
            }
          }

          CheckedListComboBoxItem allItem1 = (CheckedListComboBoxItem)parent.Items[0];
          comrec = new COMRECT(0, 0, parent.ItemHeight, parent.ItemHeight);
          if (!checkedAll)
          {
            parent.RaiseCheckStateChanged(parent, new ItemCheckEventArgs(0, CheckState.Indeterminate, allItem1.CheckState));
            allItem1.CheckState = CheckState.Indeterminate;
            InvalidateRect(this.Handle, comrec, false);
          }
          else if (allItem1.CheckState != CheckState.Checked)
          {
            parent.RaiseCheckStateChanged(parent, new ItemCheckEventArgs(0, CheckState.Checked, allItem1.CheckState));
            allItem1.CheckState = CheckState.Checked;
            InvalidateRect(this.Handle, comrec, false);
          }
          break; 
      }
    }
    parent.RaiseCheckStateChanged(parent, new ItemCheckEventArgs(index, chState, item.CheckState));
    item.CheckState = chState;

    comrec = index == 0 ? new COMRECT(0, 0, parent.ItemHeight, parent.ItemHeight * parent.Items.Count) : new COMRECT(0, index * parent.ItemHeight, parent.ItemHeight, (index + 1) * parent.ItemHeight);
    InvalidateRect(this.Handle, comrec, false);
  }

  private readonly CheckedListComboBox parent;
  public void Update()
  {
    UpdateWindow(this.Handle);
  }
}

public class CheckedListComboBox : ComboBox
{
  // Events
  public event ItemCheckEventHandler CheckStateChanged;
  // Methods
  public CheckedListComboBox()
  {
    base.DrawMode = DrawMode.OwnerDrawFixed;
    base.DropDownStyle = ComboBoxStyle.DropDownList;
    base.DoubleBuffered = true;
    base.Items.Add(new CheckedListComboBoxItem("Select All", CheckState.Unchecked));
  }
  private double dpiScale = 1f;
  protected override void ScaleControl(SizeF factor, BoundsSpecified specified)
  {
    dpiScale = factor.Width;
    base.ScaleControl(factor, specified);
  }

  public static void Sort(List<CheckedListComboBoxItem> array)
  {
    int endIndex = array.Count - 1;
    for (int i = 0; i <= endIndex; i++)
    {
      for (int j = i + 1; j <= endIndex; j++)
      {
        if ((array[i].Text.Length > array[j].Text.Length))
        {
          CheckedListComboBoxItem temp = array[i];
          array[i] = array[j];
          array[j] = temp;
        }
      }
    }
  }

  protected override void OnKeyDown(KeyEventArgs e)
  {
    switch (e.KeyCode)
    {
      case Keys.Space:
        if (DroppedDown && SelectedItem is CheckedListComboBoxItem)
        {
          dropDown.CheckItem(SelectedIndex);
        }
        break;
      case Keys.Down:
        if (!DroppedDown)
        {
          DroppedDown = true;
          SelectedIndex = -1;
          return;
        }
        break;
    }
    base.OnKeyDown(e);
  }
  internal void RaiseCheckStateChanged(object sender, ItemCheckEventArgs e)
  {
    if (CheckStateChanged != null)
    {
      CheckStateChanged(sender, e);
    }
  }

  protected override void OnDrawItem(DrawItemEventArgs e)
  {
    if (!(dpiScale == 1f))
    {
      this.SuspendLayout();
      this.ItemHeight =(int) (this.ItemHeight * dpiScale);
      this.ResumeLayout(false);
    }
    Font font = new Font(this.Font.FontFamily, Convert.ToSingle(this.Font.Size * dpiScale), this.Font.Style, this.Font.Unit);
    Size size = CheckBoxRenderer.GetGlyphSize(e.Graphics, CheckBoxState.UncheckedNormal);
    int checkBoxHeight = Convert.ToInt32((this.ItemHeight - size.Height) / 2);
    int textHeight = checkBoxHeight - 1;
    if ((e.Index != -1))
    {
      if (!DroppedDown | e.State == DrawItemState.ComboBoxEdit)
      {
        SolidBrush br = new SolidBrush(SystemColors.WindowText);
        e.Graphics.DrawString(this.displayTextField, font, br, new PointF(e.Bounds.X, e.Bounds.Y + textHeight));
        br.Dispose();
        if (Focused)
        {
          e.DrawFocusRectangle();
        }
        return;
      }
      e.DrawBackground();
      if (! (base.Items[e.Index] is CheckedListComboBoxItem))
      {
        SolidBrush br = new SolidBrush(SystemColors.WindowText);
        e.Graphics.DrawString(base.Items[e.Index].ToString(), font, br, new PointF(e.Bounds.X, e.Bounds.Y + textHeight));
        br.Dispose();
      }
      else
      {
        CheckedListComboBoxItem box = (CheckedListComboBoxItem)base.Items[e.Index];
        CheckBoxRenderer.RenderMatchingApplicationState = true;
        CheckBoxState state = default(CheckBoxState);
        switch (box.CheckState)
        {
          case CheckState.Checked:
            state = CheckBoxState.CheckedNormal;
            break; 
          case CheckState.Unchecked:
            state = CheckBoxState.UncheckedNormal;
            break; 
          case CheckState.Indeterminate:
            state = CheckBoxState.MixedNormal;
            break; 
        }
        CheckBoxRenderer.DrawCheckBox(e.Graphics, new Point(e.Bounds.X + checkBoxHeight, e.Bounds.Y + checkBoxHeight), state);
        SolidBrush br = null;
        if ((e.State & DrawItemState.Focus) == DrawItemState.Focus)
        {
          br = new SolidBrush(SystemColors.HighlightText);
        }
        else
        {
          br = new SolidBrush(SystemColors.WindowText);
        }
        e.Graphics.DrawString(box.Text, font, br, new PointF(e.Bounds.X + checkBoxHeight + e.Bounds.Height, e.Bounds.Y + checkBoxHeight));
        br.Dispose();
      }
    }
    else
    {
      SolidBrush br = new SolidBrush(SystemColors.WindowText);
      e.Graphics.DrawString(this.displayTextField, font, br, new PointF(e.Bounds.X, e.Bounds.Y + textHeight));
      br.Dispose();
      if (Focused)
      {
        e.DrawFocusRectangle();
      }
    }
  }
  
  public string DisplayText
  {
    get { return displayTextField; }
    set
    {
      displayTextField = value;
      Invalidate();
    }
  }
  protected override void WndProc(ref Message m)
  {
    base.WndProc(ref m);
    if ((m.Msg == Convert.ToInt32(Msg.WM_PARENTNOTIFY)))
    {
      long loWord =(long) m.WParam & 0xffffL;
      if (loWord == (long)Msg.WM_CREATE)
      {
        if (dropDown == null)
        {
          dropDown = new ListBoxNative(this);
        }
        dropDown.AssignHandle(m.LParam);
      }
    }
  }
  // Fields
  private string displayTextField;
  private ListBoxNative dropDown;
}


public class CheckedListComboBoxItem
{
  // Methods
  public CheckedListComboBoxItem(string text, System.Windows.Forms.CheckState initialCheckState)
  {
    this._checkState = initialCheckState;
    this._text = text;
  }
  public CheckState CheckState
  {
    get { return this._checkState; }
    set
    {
      if (value != _checkState)
      {
        this._checkState = value;
      }
    }
  }

  public object Tag
  {
    get { return this._tag; }
    set { this._tag = value; }
  }

  public string Text
  {
    get { return this._text; }
    set { this._text = value; }
  }
  // Fields
  private CheckState _checkState = CheckState.Unchecked;
  private object _tag = null;
  private string _text = "";
}


