﻿using FarPoint.Win.Spread;
using FarPoint.Win.Spread.CellType;
using GrapeCity.Spreadsheet;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SpreadWinDemo.Formulas
{
  public partial class Formulas : Form
  {
    public Formulas()
    {
      InitializeComponent();
    }

    public class TaxValueFunctionInfo : FarPoint.CalcEngine.FunctionInfo
    {
      public override string Name { get { return "TAXVALUE"; } }
      public override int MinArgs { get { return 1; } }
      public override int MaxArgs { get { return 2; } }
      public override object Evaluate(object[] args)
      {
        double num = FarPoint.CalcEngine.CalcConvert.ToDouble(args[0]);
        double taxrate = args.Length > 1 ? FarPoint.CalcEngine.CalcConvert.ToDouble(args[1]) : 0.15;

        return num - (num * taxrate);
      }
    }
    private void InitializeSubtotal()
    {
      // Range group
      SheetView sheet = FpSpread1.Sheets[4];

      sheet.ColumnCount = 500;
      sheet.RowCount = 500;
      sheet.Columns[0].Width = sheet.Columns[1].Width = 80;
      sheet.Columns[2].Width = 110;
      sheet.AddSelection(0, 0, 1, 1);
      // set value
      object[,] t = new object[,]
                             {
                               {"= Eastern ==========", "", "", 0, 0, "", 0},
                               {"Eastern", "Atlantic", "Celtics", 57, 19, "-", 0.750},
                               {"Eastern", "Atlantic", "76ers", 38, 35, 17.5, 0.521},
                               {"Eastern", "Atlantic", "Nets", 31, 44, 25.5, 0.413},
                               {"Eastern", "Atlantic", "Raptors", 29, 45, 27, 0.392},
                               {"Eastern", "Atlantic", "Knicks", 29, 46, 27.5, 0.387},
                               {"Eastern", "Central", "Cavaliers", 61, 13, "-", 0.824},
                               {"Eastern", "Central", "Pistons", 36, 39, 25.5, 0.480},
                               {"Eastern", "Central", "Bulls", 36, 40, 26, 0.474},
                               {"Eastern", "Central", "Pacers", 32, 43, 29.5, 0.427},
                               {"Eastern", "Central", "Bucks", 32, 44, 30, 0.421},
                               {"Eastern", "Southeast", "Magic", 55, 19, "-", 0.743},
                               {"Eastern", "Southeast", "Hawks", 43, 32, 12.5, 0.573},
                               {"Eastern", "Southeast", "Heat", 39, 36, 16.5, 0.520},
                               {"Eastern", "Southeast", "Bobcats", 34, 41, 21.5, 0.453},
                               {"Eastern", "Southeast", "Wizards", 17, 59, 39, 0.224},
                               {"= Total ==========", "", "", 0, 0, "", 0},
                               {"= Western ==========", "", "", 0, 0, "", 0},
                               {"Western", "Northwest", "Nuggets", 49, 26, "-", 0.653},
                               {"Western", "Northwest", "Trail Blazers", 47, 27, 1.5, 0.635},
                               {"Western", "Northwest", "Jazz", 46, 28, 2.5, 0.622},
                               {"Western", "Northwest", "Thunder", 21, 53, 27.5, 0.284},
                               {"Western", "Northwest", "Timberwolves", 21, 54, 28, 0.280},
                               {"Western", "Pacific", "Lakers", 59, 16, "-", 0.787},
                               {"Western", "Pacific", "Suns", 41, 34, 18, 0.547},
                               {"Western", "Pacific", "Warriors", 26, 49, 33, 0.347},
                               {"Western", "Pacific", "Clippers", 18, 57, 41, 0.240},
                               {"Western", "Pacific", "Kings", 16, 58, 42.5, 0.216},
                               {"Western", "Southwest", "Spurs", 48, 29, "-", 0.649},
                               {"Western", "Southwest", "Rockets", 48, 27, 0.5, 0.640},
                               {"Western", "Southwest", "Hornets", 47, 27, 1, 0.635},
                               {"Western", "Southwest", "Mavericks", 45, 30, 3.5, 0.600},
                               {"Western", "Southwest", "Grizzlies", 20, 54, 28, 0.270},
                               {"= Total ==========", "", "", 0, 0, "", 0},
                             };

      for (int r = 0; r <= t.GetUpperBound(0); r++)
      {
        for (int c = 0; c <= t.GetUpperBound(1); c++)
        {
          sheet.SetValue(r, c, t[r, c]);
        }
      }
      sheet.Cells[0, 0].ColumnSpan = 7;
      sheet.Cells[16, 0].ColumnSpan = 3;
      sheet.Cells[17, 0].ColumnSpan = 7;
      sheet.Cells[33, 0].ColumnSpan = 3;
      sheet.ColumnHeader.RowCount = 2;
      sheet.ColumnHeader.AutoTextIndex = 1;
      sheet.ColumnHeader.Cells[0, 0].Value = "2008-09 NBA Regular Season Standings";
      sheet.ColumnHeader.Cells[0, 0].ColumnSpan = 7;
      sheet.ColumnHeader.Cells[0, 0].Font = new System.Drawing.Font(new System.Drawing.FontFamily("Arial"), 14);

      sheet.ColumnHeader.Cells[0, 0].HorizontalAlignment = CellHorizontalAlignment.Center;
      sheet.ColumnHeader.Cells[0, 0].VerticalAlignment = CellVerticalAlignment.Center;
      sheet.ColumnHeader.Cells[0, 0].ForeColor = System.Drawing.Color.Gray;
      sheet.ColumnHeader.Rows[0].Height = 30;
      sheet.Columns[0].Label = "Conference";
      sheet.Columns[1].Label = "Standing";
      sheet.Columns[2].Label = "Team";
      sheet.Columns[3].Label = "W";
      sheet.Columns[4].Label = "L";
      sheet.Columns[5].Label = "GB";
      sheet.Columns[6].Label = "PCT";

      // set row range group
      sheet.AddRangeGroup(1, 15, true); // eastern
      sheet.AddRangeGroup(1, 4, true);
      sheet.AddRangeGroup(6, 4, true);
      sheet.AddRangeGroup(11, 4, true);
      sheet.AddRangeGroup(18, 15, true); // western
      sheet.AddRangeGroup(18, 4, true);
      sheet.AddRangeGroup(23, 4, true);
      sheet.AddRangeGroup(28, 4, true);
      sheet.Cells["G34"].Formula = "SUBTOTAL(109,G19:G33)";
      sheet.Cells["G17"].Formula = "SUBTOTAL(109,G1:G16)";
      //Add line border
      sheet.Cells["G34"].Border = new FarPoint.Win.LineBorder(System.Drawing.Color.Red, 2);
      //Add border around total due cell
      sheet.Cells["G17"].Border = new FarPoint.Win.LineBorder(System.Drawing.Color.Red, 2);

      TableView table = sheet.AddTable(35, 2, 7, 5);
      table.TotalRowVisible = true;
      int firstRow = table.HeaderRowIndex;
      int firstColumn = table.DataRange.Column;
      sheet.Cells.Get(firstRow, 2).Value = "SalesPers";
      sheet.Cells.Get(firstRow, 3).Value = "Region";
      sheet.Cells.Get(firstRow, 4).Value = "SaleAmt";
      sheet.Cells.Get(firstRow, 5).Value = "ComPct";
      sheet.Cells.Get(firstRow, 6).Value = "ComAmt";

      sheet.Cells[firstRow + 1, firstColumn].Value = "Joe";
      sheet.Cells[firstRow + 2, firstColumn].Value = "Robert";
      sheet.Cells[firstRow + 3, firstColumn].Value = "Michelle";
      sheet.Cells[firstRow + 4, firstColumn].Value = "Erich";
      sheet.Cells[firstRow + 5, firstColumn].Value = "Dafna";
      sheet.Cells[firstRow + 6, firstColumn].Value = "Rob";

      sheet.Cells[firstRow + 1, firstColumn + 1].Value = "North";
      sheet.Cells[firstRow + 2, firstColumn + 1].Value = "South";
      sheet.Cells[firstRow + 3, firstColumn + 1].Value = "East";
      sheet.Cells[firstRow + 4, firstColumn + 1].Value = "West";
      sheet.Cells[firstRow + 5, firstColumn + 1].Value = "North";
      sheet.Cells[firstRow + 6, firstColumn + 1].Value = "South";

      sheet.Cells[firstRow + 1, firstColumn + 2].ParseFormatString = "C";
      sheet.Cells[firstRow + 1, firstColumn + 2].Value = 260;
      sheet.Cells[firstRow + 2, firstColumn + 2].ParseFormatString = "C";
      sheet.Cells[firstRow + 2, firstColumn + 2].Value = 660;
      sheet.Cells[firstRow + 3, firstColumn + 2].ParseFormatString = "C";
      sheet.Cells[firstRow + 3, firstColumn + 2].Value = 940;
      sheet.Cells[firstRow + 4, firstColumn + 2].ParseFormatString = "C";
      sheet.Cells[firstRow + 4, firstColumn + 2].Value = 410;
      sheet.Cells[firstRow + 5, firstColumn + 2].ParseFormatString = "C";
      sheet.Cells[firstRow + 5, firstColumn + 2].Value = 800;
      sheet.Cells[firstRow + 6, firstColumn + 2].ParseFormatString = "C";
      sheet.Cells[firstRow + 6, firstColumn + 2].Value = 900;

      sheet.Cells[firstRow + 1, firstColumn + 3].Value = 10;
      sheet.Cells[firstRow + 2, firstColumn + 3].Value = 15;
      sheet.Cells[firstRow + 3, firstColumn + 3].Value = 15;
      sheet.Cells[firstRow + 4, firstColumn + 3].Value = 12;
      sheet.Cells[firstRow + 5, firstColumn + 3].Value = 14;
      sheet.Cells[firstRow + 6, firstColumn + 3].Value = 15;


      sheet.Cells[firstRow + 1, firstColumn + 4].Value = 26;
      sheet.Cells[firstRow + 2, firstColumn + 4].Value = 99;
      sheet.Cells[firstRow + 3, firstColumn + 4].Value = 25;
      sheet.Cells[firstRow + 4, firstColumn + 4].Value = 14;
      sheet.Cells[firstRow + 5, firstColumn + 4].Value = 32;
      sheet.Cells[firstRow + 6, firstColumn + 4].Value = 34;

      sheet.Cells[table.TotalRowIndex, firstColumn + 1].Formula = "SUBTOTAL(103,[Region])";
      sheet.Cells[table.TotalRowIndex, firstColumn + 3].Formula = "SUBTOTAL(101,[ComPct])";
    }

    private void InitializeArrayFormula()
    {
      FpSpread1.Sheets[5].Protect = true;
      FpSpread1.Sheets[5].RowCount = 10;
      FpSpread1.Sheets[5].ColumnCount = 10;
      FpSpread1.Sheets[5].SetClip(0, 0, 1, 6, "Customer\tProduct\tSale number ( items)\tPrice ($)\tTotal no tax ($)\tTotal with tax ($)");
      FpSpread1.Sheets[5].SetClip(1, 0, 8, 2, "Mike\tApple\nSally\tLemon\nNancy\tLemon\nJane\tLemon\nSarah\tApple\nLucy\tApple\nEric\tLemon\nTom\tApple");
      FpSpread1.Sheets[5].Cells["C2:C9"].FormulaArray = "{2;10;100;19;34;15;20;14}";
      FpSpread1.Sheets[5].Cells["D2:D9"].Formula = @"IFS(B2=""Apple"",10,B2=""Lemon"",20)";

      FpSpread1.Sheets[5].Cells["D2"].Note = @"Excel 2016 don't support IFS function so if want to export to Excel 2016, pls use formula is IF(B2=""Apple"",10,(IF(B2=""Lemon"",20,"""")))";
      FpSpread1.Sheets[5].Cells["E2:E9"].Formula = "C2*D2";
      FpSpread1.Sheets[5].Cells["F2:F9"].FormulaArray = "C2:C9*D2:D9*1.08";
      FpSpread1.Sheets[5].Cells["E10"].Text = "Totals";
      FpSpread1.Sheets[5].Cells["F10"].Formula = "SUM(F2:F9)";

      FpSpread1.Sheets[5].Cells["D2:F9"].CellType = new FarPoint.Win.Spread.CellType.CurrencyCellType();
       FpSpread1.Sheets[5].Cells["F10"].CellType = new FarPoint.Win.Spread.CellType.CurrencyCellType();
      FpSpread1.Sheets[5].Cells["A1:F1"].BackColor = System.Drawing.Color.LightGreen;
      FpSpread1.Sheets[5].Cells["A2:B9"].BackColor = System.Drawing.Color.LightGreen;
      FpSpread1.Sheets[5].Cells["F2:F10"].BackColor = System.Drawing.Color.LightGreen;
      FpSpread1.Sheets[5].Cells["E10"].BackColor = System.Drawing.Color.LightGreen;

      FpSpread1.Sheets[5].Cells["C2:E9"].BackColor = System.Drawing.Color.LightYellow;
      FpSpread1.Sheets[5].Cells["I1:J4"].BackColor = System.Drawing.Color.LightYellow;
      FpSpread1.Sheets[5].Columns.Default.Width = 90;
      FpSpread1.Sheets[5].Columns[6].Width = 30;
      FpSpread1.Sheets[5].Columns[2].Width = 110;
      FpSpread1.Sheets[5].Columns[7].Width = 130;

      FpSpread1.Sheets[5].Cells["I1"].Text = @"Customer";
      FpSpread1.Sheets[5].Cells["J1"].Text = @"Product";

      FpSpread1.Sheets[5].Cells["H3"].Text = @"Total sale number (items)";
      FpSpread1.Sheets[5].Cells["H4"].Text = @"Total sale with tax ($)";
      GrapeCity.Spreadsheet.DataValidation dv1 = FpSpread1.Sheets[5].Validations.Add(1, 8, 1, 8, GrapeCity.Spreadsheet.DataValidationType.List,  DataValidationErrorStyle.Warning,GrapeCity.Spreadsheet.DataValidationOperator.Equal, "=$A$2:$A$9", null);
      GrapeCity.Spreadsheet.DataValidation dv2 = FpSpread1.Sheets[5].Validations.Add(1, 9, 1, 9, GrapeCity.Spreadsheet.DataValidationType.List, DataValidationErrorStyle.Warning,GrapeCity.Spreadsheet.DataValidationOperator.Equal,  "Apple, Lemon", null);


      FpSpread1.Sheets[5].Cells["I3"].Formula = "SUMIF($A$2:$A$9,$I$2,$C$2:$C$9)";
      FpSpread1.Sheets[5].Cells["I4"].FormulaArray = "SUMIF($A$2:$A$9,$I$2,$F$2:$F$9)";


      FpSpread1.Sheets[5].Cells["J3"].Formula = "SUMIF($B$2:$B$9,$J$2,$C$2:$C$9)";
      FpSpread1.Sheets[5].Cells["J4"].FormulaArray = "SUMIF($B$2:$B$9,$J$2,$F$2:$F$9)";


      FpSpread1.Sheets[5].Cells["I3:J4"].Locked = true;
      FpSpread1.Sheets[5].Cells["I4:J4"].CellType = new FarPoint.Win.Spread.CellType.CurrencyCellType();

    }
    private void Fomulas_Load(object sender, EventArgs e)
    {
      InitSpread();
      SetFormulas();
      InitializeSubtotal();
      InitializeArrayFormula();
      int i = 0, w = 0, h = 0;

      for (i = 0; i < FpSpread1.Sheets[0].ColumnCount; i++)
        w = w + (int)FpSpread1.Sheets[0].Columns[i].Width;
      w = w + (int)FpSpread1.Sheets[0].RowHeader.Columns[0].Width;

      //for (i = 0; i < FpSpread1.Sheets[0].RowCount; i++)
      //  h = h + (int)FpSpread1.Sheets[0].Rows[i].Height;
      //h = h + (int)FpSpread1.Sheets[0].ColumnHeader.Rows[0].Height;

      ////FpSpread1.Height = h + 80;
      ////FpSpread1.Width = w + 140;
      //FpSpread1.MinimumSize = new System.Drawing.Size(120, 160);

      formulaTextBox1.BackColor = System.Drawing.Color.WhiteSmoke;
      formulaTextBox1.Width = w + 75;

    }

    public void SetFormulas()
    {
      FarPoint.Win.Spread.CellType.CurrencyCellType curType = new FarPoint.Win.Spread.CellType.CurrencyCellType();
      //Set up the formulas

      FpSpread1.AllowUserFormulas = true;
      //Set up celltypes for formula cells
      curType.DecimalPlaces = 2;
      curType.FixedPoint = true;
      curType.LeadingZero = FarPoint.Win.Spread.CellType.LeadingZero.UseRegional;


      for (int i = 0; i < 3; i++)
      {
        FpSpread1.Sheets[i].Columns[1].CellType = curType;
        FpSpread1.Sheets[i].Cells[4, 1, 4, 4].CellType = curType;

        //Set row formulas
        FpSpread1.Sheets[i].Cells[0, 1].Formula = "SUM(C1:E1)";
        FpSpread1.Sheets[i].Cells[1, 1].Formula = "SUM(C2:E2)";
        FpSpread1.Sheets[i].Cells[2, 1].Formula = "SUM(C3:E3)";
        FpSpread1.Sheets[i].Cells[3, 1].Formula = "SUM(C4:E4)";
        //Set column formulas
        FpSpread1.Sheets[i].Cells[4, 1].Formula = "SUM(B1:B4)";
        FpSpread1.Sheets[i].Cells[4, 2].Formula = "SUM(C1:C4)";
        FpSpread1.Sheets[i].Cells[4, 3].Formula = "SUM(D1:D4)";
        FpSpread1.Sheets[i].Cells[4, 4].Formula = "SUM(E1:E4)";

        FpSpread1.Sheets[i].Cells[0, 1].Note = "SUM(C1:E1)";
        FpSpread1.Sheets[i].Cells[1, 1].Note = "SUM(C2:E2)";
        FpSpread1.Sheets[i].Cells[2, 1].Note = "SUM(C3:E3)";
        FpSpread1.Sheets[i].Cells[3, 1].Note = "SUM(C4:E4)";
        //Set column formulas
        FpSpread1.Sheets[i].Cells[4, 1].Note = "SUM(B1:B4)";
        FpSpread1.Sheets[i].Cells[4, 2].Note = "SUM(C1:C4)";
        FpSpread1.Sheets[i].Cells[4, 3].Note = "SUM(D1:D4)";
        FpSpread1.Sheets[i].Cells[4, 4].Note = "SUM(E1:E4)";
        //FpSpread1.Sheets[i].FrozenTrailingRowCount = 1;
        //FpSpread1.Sheets[i].FrozenTrailingRowCount = 1;
      }

      FpSpread1.Sheets[3].Columns[1].CellType = curType;
      FpSpread1.Sheets[3].Cells[4, 1, 4, 4].CellType = curType;

      //Set row formulas
      FpSpread1.Sheets[3].Cells[0, 1].Formula = "SUM(CustomerA!C1, CustomerA!D1, CustomerA!E1, CustomerB!C1,CustomerB!D1,CustomerB!E1, CustomerC!C1,CustomerC!D1,CustomerC!E1)";
      FpSpread1.Sheets[3].Cells[1, 1].Formula = "SUM(CustomerA!C2, CustomerA!D2, CustomerA!E2,CustomerB!C2, CustomerB!D2, CustomerB!E2,CustomerB!C2, CustomerB!D2, CustomerA!B2)";
      FpSpread1.Sheets[3].Cells[2, 1].Formula = "SUM(CustomerA!C3, CustomerA!D3, CustomerA!E3,CustomerB!C3, CustomerB!D3, CustomerB!E3,CustomerB!C3, CustomerB!D3, CustomerA!B3)";
      FpSpread1.Sheets[3].Cells[3, 1].Formula = "SUM(CustomerA!C4, CustomerA!D4, CustomerA!E4,CustomerB!C4, CustomerB!D4, CustomerB!E4,CustomerB!C4, CustomerB!D4, CustomerA!B4)";
      //Set column formulas
      FpSpread1.Sheets[3].Cells[4, 1].Formula = "SUM(CustomerA!B1, CustomerA!B2, CustomerA!B3, CustomerA!B4, CustomerB!B1, CustomerB!B2, CustomerB!B3, CustomerB!B4,CustomerC!B1, CustomerC!B2, CustomerC!B3, CustomerC!B4)";
      FpSpread1.Sheets[3].Cells[4, 2].Formula = "SUM(CustomerA!C1, CustomerA!C2, CustomerA!C3, CustomerA!C4, CustomerB!C1, CustomerB!C2, CustomerB!C3, CustomerB!C4,CustomerC!C1, CustomerC!C2, CustomerC!C3, CustomerC!C4)";
      FpSpread1.Sheets[3].Cells[4, 3].Formula = "SUM(CustomerA!D1, CustomerA!D2, CustomerA!D3, CustomerA!D4, CustomerB!D1, CustomerB!D2, CustomerB!D3, CustomerB!D4,CustomerC!D1, CustomerC!D2, CustomerC!D3, CustomerC!D4)";
      FpSpread1.Sheets[3].Cells[4, 4].Formula = "SUM(CustomerA!E1, CustomerA!E2, CustomerA!E3, CustomerA!E4, CustomerB!E1, CustomerB!E2, CustomerB!E3, CustomerB!E4,CustomerC!E1, CustomerC!E2, CustomerC!E3, CustomerC!E4)";

      FpSpread1.Sheets[3].Cells[0, 2].Formula = "SUM(CustomerA!C1, CustomerB!C1, CustomerC!C1)";
      FpSpread1.Sheets[3].Cells[0, 3].Formula = "SUM(CustomerA!D1, CustomerB!D1, CustomerC!D1)";
      FpSpread1.Sheets[3].Cells[0, 4].Formula = "SUM(CustomerA!E1, CustomerB!E1, CustomerC!E1)";
      FpSpread1.Sheets[3].Cells[1, 2].Formula = "SUM(CustomerA!C2, CustomerB!C2, CustomerC!C2)";
      FpSpread1.Sheets[3].Cells[1, 3].Formula = "SUM(CustomerA!D2, CustomerB!D2, CustomerC!D2)";
      FpSpread1.Sheets[3].Cells[1, 4].Formula = "SUM(CustomerA!E2, CustomerB!E2, CustomerC!E2)";
      FpSpread1.Sheets[3].Cells[2, 2].Formula = "SUM(CustomerA!C3, CustomerB!C3, CustomerC!C3)";
      FpSpread1.Sheets[3].Cells[2, 3].Formula = "SUM(CustomerA!D3, CustomerB!D3, CustomerC!D3)";
      FpSpread1.Sheets[3].Cells[2, 4].Formula = "SUM(CustomerA!E3, CustomerB!E3, CustomerC!E3)";
      FpSpread1.Sheets[3].Cells[3, 2].Formula = "SUM(CustomerA!C4, CustomerB!C4, CustomerC!C4)";
      FpSpread1.Sheets[3].Cells[3, 3].Formula = "SUM(CustomerA!D4, CustomerB!D4, CustomerC!D4)";
      FpSpread1.Sheets[3].Cells[3, 4].Formula = "SUM(CustomerA!E4, CustomerB!E4, CustomerC!E4)";

      FpSpread1.Sheets[3].Cells[0, 2].Note = "SUM(CustomerA!C1, CustomerB!C1, CustomerC!C1)";
      FpSpread1.Sheets[3].Cells[0, 3].Note = "SUM(CustomerA!D1, CustomerB!D1, CustomerC!D1)";
      FpSpread1.Sheets[3].Cells[0, 4].Note = "SUM(CustomerA!E1, CustomerB!E1, CustomerC!E1)";
      FpSpread1.Sheets[3].Cells[1, 2].Note = "SUM(CustomerA!C2, CustomerB!C2, CustomerC!C2)";
      FpSpread1.Sheets[3].Cells[1, 3].Note = "SUM(CustomerA!D2, CustomerB!D2, CustomerC!D2)";
      FpSpread1.Sheets[3].Cells[1, 4].Note = "SUM(CustomerA!E2, CustomerB!E2, CustomerC!E2)";
      FpSpread1.Sheets[3].Cells[2, 2].Note = "SUM(CustomerA!C3, CustomerB!C3, CustomerC!C3)";
      FpSpread1.Sheets[3].Cells[2, 3].Note = "SUM(CustomerA!D3, CustomerB!D3, CustomerC!D3)";
      FpSpread1.Sheets[3].Cells[2, 4].Note = "SUM(CustomerA!E3, CustomerB!E3, CustomerC!E3)";
      FpSpread1.Sheets[3].Cells[3, 2].Note = "SUM(CustomerA!C4, CustomerB!C4, CustomerC!C4)";
      FpSpread1.Sheets[3].Cells[3, 3].Note = "SUM(CustomerA!D4, CustomerB!D4, CustomerC!D4)";
      FpSpread1.Sheets[3].Cells[3, 4].Note = "SUM(CustomerA!E4, CustomerB!E4, CustomerC!E4)";

      FpSpread1.Sheets[3].AddCustomFunction(new TaxValueFunctionInfo());
      FpSpread1.Sheets[3].Cells[4, 0].Formula = "TAXVALUE(B5, A2)";
      FpSpread1.Sheets[3].Cells[4, 0].Note = "TAXVALUE(B5, A2)";
      FpSpread1.Sheets[3].Cells[1, 0].Value = .15;
      FpSpread1.Sheets[3].Cells[0, 0].Text = "Tax Rate";

      //Set row formulas notes
      FpSpread1.Sheets[3].Cells[0, 1].Note = "SUM(CustomerA!C1, CustomerA!D1, CustomerA!E1, CustomerB!C1,CustomerB!D1,CustomerB!E1, CustomerC!C1,CustomerC!D1,CustomerC!E1)";
      FpSpread1.Sheets[3].Cells[1, 1].Note = "SUM(CustomerA!C2, CustomerA!D2, CustomerA!E2,CustomerB!C2, CustomerB!D2, CustomerB!E2,CustomerB!C2, CustomerB!D2, CustomerA!B2)";
      FpSpread1.Sheets[3].Cells[2, 1].Note = "SUM(CustomerA!C3, CustomerA!D3, CustomerA!E3,CustomerB!C3, CustomerB!D3, CustomerB!E3,CustomerB!C3, CustomerB!D3, CustomerA!B3)";
      FpSpread1.Sheets[3].Cells[3, 1].Note = "SUM(CustomerA!C4, CustomerA!D4, CustomerA!E4,CustomerB!C4, CustomerB!D4, CustomerB!E4,CustomerB!C4, CustomerB!D4, CustomerA!B4)";
      //Set column formulas notes
      FpSpread1.Sheets[3].Cells[4, 1].Note = "SUM(CustomerA!B1, CustomerA!B2, CustomerA!B3, CustomerA!B4, CustomerB!B1, CustomerB!B2, CustomerB!B3, CustomerB!B4,CustomerC!B1, CustomerC!B2, CustomerC!B3, CustomerC!B4)";
      FpSpread1.Sheets[3].Cells[4, 2].Note = "SUM(CustomerA!C1, CustomerA!C2, CustomerA!C3, CustomerA!C4, CustomerB!C1, CustomerB!C2, CustomerB!C3, CustomerB!C4,CustomerC!C1, CustomerC!C2, CustomerC!C3, CustomerC!C4)";
      FpSpread1.Sheets[3].Cells[4, 3].Note = "SUM(CustomerA!D1, CustomerA!D2, CustomerA!D3, CustomerA!D4, CustomerB!D1, CustomerB!D2, CustomerB!D3, CustomerB!D4,CustomerC!D1, CustomerC!D2, CustomerC!D3, CustomerC!D4)";
      FpSpread1.Sheets[3].Cells[4, 4].Note = "SUM(CustomerA!E1, CustomerA!E2, CustomerA!E3, CustomerA!E4, CustomerB!E1, CustomerB!E2, CustomerB!E3, CustomerB!E4,CustomerC!E1, CustomerC!E2, CustomerC!E3, CustomerC!E4)";
      // FpSpread1.Sheets[3].FrozenTrailingRowCount = 1;

    }
    public void InitSpread()
    {
      FpSpread1.TabStripInsertTab = false;
      FpSpread1.HorizontalScrollBarPolicy = FarPoint.Win.Spread.ScrollBarPolicy.Never;
      FpSpread1.Sheets.Count = 6;
      FpSpread1.Sheets[0].SheetName = "CustomerA";
      FpSpread1.Sheets[1].SheetName = "CustomerB";
      FpSpread1.Sheets[2].SheetName = "CustomerC";
      FpSpread1.Sheets[3].SheetName = "Totals Sheet";
      FpSpread1.Sheets[4].SheetName = "SubTotal";
      FpSpread1.Sheets[5].SheetName = "ArrayFormula";
      for (int i = 0; i <= 3; i++)
      {
        FpSpread1.Sheets[i].ColumnHeader.DefaultStyle.Font = new System.Drawing.Font("Verdana", 8);
        FpSpread1.Sheets[i].RowHeader.DefaultStyle.Font = new System.Drawing.Font("Verdana", 8);
        FpSpread1.Sheets[i].DefaultStyle.Font = new System.Drawing.Font("Verdana", 8);
        FpSpread1.Sheets[i].ColumnCount = 5;
        FpSpread1.Sheets[i].RowCount = 5;

        //Set up the headers
        FpSpread1.Sheets[i].ColumnHeader.RowCount = 2;
        FpSpread1.Sheets[i].Models.ColumnHeaderSpan.Add(0, 0, 2, 1);
        if (i != 3)
          FpSpread1.Sheets[i].ColumnHeader.Cells[0, 0].Text = "Invoice";
        //.ColumnHeader.Cells(1, 1).Text = "Order #"
        //.ColumnHeader.Cells(1, 2).Text = "Date"
        FpSpread1.Sheets[i].Models.ColumnHeaderSpan.Add(0, 1, 2, 1);
        FpSpread1.Sheets[i].ColumnHeader.Cells[0, 1].Text = "Total Due";
        FpSpread1.Sheets[i].Models.ColumnHeaderSpan.Add(0, 2, 1, 3);
        FpSpread1.Sheets[i].ColumnHeader.Cells[0, 2].Text = "Aged Receivables";
        FpSpread1.Sheets[i].ColumnHeader.Cells[0, 2].Font = new System.Drawing.Font("Veranda", 8, System.Drawing.FontStyle.Italic | System.Drawing.FontStyle.Bold);
        FpSpread1.Sheets[i].ColumnHeader.Cells[1, 2].Text = "0-30";
        FpSpread1.Sheets[i].ColumnHeader.Cells[1, 3].Text = "31-60";
        FpSpread1.Sheets[i].ColumnHeader.Cells[1, 4].Text = "61+";
        //Set up Columns styles
        int fpwid = 83;
        FpSpread1.Sheets[i].Columns[1].BackColor = System.Drawing.Color.NavajoWhite;
        FpSpread1.Sheets[i].Columns[2, 4].BackColor = System.Drawing.Color.Gainsboro;
        FpSpread1.Sheets[i].Columns[0].Width = 65;
        FpSpread1.Sheets[i].Columns[1].Width = fpwid;
        FpSpread1.Sheets[i].Columns[3].Width = fpwid;
        FpSpread1.Sheets[i].Columns[3].Width = fpwid;
        FpSpread1.Sheets[i].Columns[4].Width = fpwid;
        //Add line border
        FpSpread1.Sheets[i].Cells[4, 1, 4, 4].Border = new FarPoint.Win.LineBorder(System.Drawing.Color.Red, 2);
        //Add border around total due cell
        FpSpread1.Sheets[i].Cells[4, 1].Border = new FarPoint.Win.LineBorder(System.Drawing.Color.Red, 2);
      }
      //Add Sample data

      //Order number
      FpSpread1.Sheets[0].Cells[0, 0].Text = "10365";
      FpSpread1.Sheets[0].Cells[1, 0].Text = "10507";
      FpSpread1.Sheets[0].Cells[2, 0].Text = "10535";
      FpSpread1.Sheets[0].Cells[3, 0].Text = "10573";
      //Add amounts
      AddAmount(0, 0, 2, 503.2);
      AddAmount(0, 0, 3, 0);
      AddAmount(0, 0, 4, 0);
      AddAmount(0, 1, 2, 0);
      AddAmount(0, 1, 3, 0);
      AddAmount(0, 1, 4, 1648.5);
      AddAmount(0, 2, 2, 0);
      AddAmount(0, 2, 3, 395.2);
      AddAmount(0, 2, 4, 0);
      AddAmount(0, 3, 2, 542.5);
      AddAmount(0, 3, 3, 0);
      AddAmount(0, 3, 4, 0);
      //Order number
      FpSpread1.Sheets[1].Cells[0, 0].Text = "11365";
      FpSpread1.Sheets[1].Cells[1, 0].Text = "11507";
      FpSpread1.Sheets[1].Cells[2, 0].Text = "11535";
      FpSpread1.Sheets[1].Cells[3, 0].Text = "11573";
      //Add amounts
      AddAmount(1, 0, 2, 801.3);
      AddAmount(1, 0, 3, 0);
      AddAmount(1, 0, 4, 0);
      AddAmount(1, 1, 2, 0);
      AddAmount(1, 1, 3, 0);
      AddAmount(1, 1, 4, 1234.9);
      AddAmount(1, 2, 2, 0);
      AddAmount(1, 2, 3, 295.6);
      AddAmount(1, 2, 4, 0);
      AddAmount(1, 3, 2, 601.21);
      AddAmount(1, 3, 3, 0);
      AddAmount(1, 3, 4, 0);
      //Order number
      FpSpread1.Sheets[2].Cells[0, 0].Text = "12365";
      FpSpread1.Sheets[2].Cells[1, 0].Text = "12507";
      FpSpread1.Sheets[2].Cells[2, 0].Text = "12535";
      FpSpread1.Sheets[2].Cells[3, 0].Text = "12573";
      //Add amounts
      AddAmount(2, 0, 2, 432.9);
      AddAmount(2, 0, 3, 0);
      AddAmount(2, 0, 4, 0);
      AddAmount(2, 1, 2, 0);
      AddAmount(2, 1, 3, 0);
      AddAmount(2, 1, 4, 2005.3);
      AddAmount(2, 2, 2, 0);
      AddAmount(2, 2, 3, 291.7);
      AddAmount(2, 2, 4, 0);
      AddAmount(2, 3, 2, 675.4);
      AddAmount(2, 3, 3, 0);
      AddAmount(2, 3, 4, 0);
    }
    public void AddAmount(int sheet, int row, int col, double amt)
    {
      FarPoint.Win.Spread.CellType.CurrencyCellType curType = new FarPoint.Win.Spread.CellType.CurrencyCellType();

      FpSpread1.Sheets[sheet].Cells[row, col].Value = amt;
      curType.DecimalPlaces = 2;
      curType.FixedPoint = true;
      curType.LeadingZero = FarPoint.Win.Spread.CellType.LeadingZero.UseRegional;
      FpSpread1.Sheets[sheet].Cells[row, col].CellType = curType;

    }

    private void checkBox1_CheckedChanged(object sender, EventArgs e)
    {
      if (((CheckBox)sender).Checked)
      {
        formulaTextBox1.Attach(FpSpread1);
        //trick to display value right after attach
        FpSpread1.StartCellEditing(null, false);
        FpSpread1.CancelCellEditing();
      }
      else
      {
        formulaTextBox1.Detach();
        formulaTextBox1.ResetText();
      }
    }

    private void nameManagerButton_Click(object sender, EventArgs e)
    {
      this.FpSpread1.ShowListCustomNameForm(this);
    }

  }
}
