﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using FarPoint.Win.Spread;
using FarPoint.Win.Spread.Model;
using FarPoint.Excel;

namespace SpreadWinDemo.ImportExportExcel
{
  public partial class ImportExportExcel : Form
  {
    public ImportExportExcel()
    {
      InitializeComponent();
      InitCheckLisFlag();
    }
    public class CCBoxItem
    {
      private int val;
      public int Value
      {
        get { return val; }
        set { val = value; }
      }

      private string text;
      public string Text
      {
        get { return text; }
        set { text = value; }
      }

      public CCBoxItem()
      {
      }

      public CCBoxItem(string text, int val)
      {
        this.text = text;
        this.val = val;
      }
      public override string ToString()
      {
        return text;
      }
    }

    private const string OPEN_SELECT_ALL = "All Excel Open Flags are Selected";
    private const string OPEN_SELECT_SOME = "Some Excel Open Flags are Selected";
    private const string OPEN_SELECT = "Select Exel Open Flags";

    private const string SAVE_SELECT_ALL = "All Excel Save Flags are Selected";
    private const string SAVE_SELECT_SOME = "Some Excel Save Flags are Selected";
    private const string SAVE_SELECT = "Select Exel Save Flags";
    private void InitCheckLisFlag()
    {
      lstOpenFlag.DisplayText = "Select Excel Open Flags";
      ExcelOpenFlags[] allValues = (ExcelOpenFlags[])Enum.GetValues(typeof(ExcelOpenFlags));
      List<CheckedListComboBoxItem> openFlags = new List<CheckedListComboBoxItem>();
      foreach (ExcelOpenFlags val in allValues)
      {
        CheckedListComboBoxItem cb = new CheckedListComboBoxItem(val.ToString(), CheckState.Unchecked);
        cb = new CheckedListComboBoxItem(val.ToString(), CheckState.Unchecked);
        cb.Tag = val;
        openFlags.Add(cb);
      }
      CheckedListComboBox.Sort(openFlags);
      lstOpenFlag.Items.AddRange(openFlags.ToArray());
      ExcelSaveFlags[] allSaveValues = (ExcelSaveFlags[])Enum.GetValues(typeof(ExcelSaveFlags));
      List<CheckedListComboBoxItem> saveFlags = new List<CheckedListComboBoxItem>();
      lstSaveFlag.DisplayText = "Select Excel Save Flags";
      foreach (ExcelSaveFlags val in allSaveValues)
      {
        if (val == ExcelSaveFlags.UseOOXMLFormat)
          continue;
        CheckedListComboBoxItem cb1 = new CheckedListComboBoxItem(val.ToString(), CheckState.Unchecked);
        cb1.Tag = val;
        saveFlags.Add(cb1);
      }
      CheckedListComboBox.Sort(saveFlags);
      lstSaveFlag.Items.AddRange(saveFlags.ToArray());
    }
    private void FillSampleData(SheetView sheet, CellRange range)
    {
      System.Random r = new System.Random();
      for (int i = 0; i < range.RowCount; i++)
      {
        for (int j = 0; j < range.ColumnCount; j++)
        {
          sheet.SetValue(range.Row + i, range.Column + j, r.Next(50, 300));
        }
      }
    }
    private void btnSaveExel_Click(object sender, EventArgs e)
    {
      SaveFileDialog saveFileDialog = new SaveFileDialog();
      saveFileDialog.Filter = "Excel 97-2003 Workbook(*.xls)|*.xls|Excel Workbook(*.xlsx)|*.xlsx";
      saveFileDialog.FilterIndex = 1;
      if (saveFileDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
      {
        string fileName = saveFileDialog.FileName;
        if (saveFileDialog.FilterIndex==2)
          this.fpSpread1.SaveExcel(fileName, GetSaveFlag() | ExcelSaveFlags.UseOOXMLFormat);
        else
          this.fpSpread1.SaveExcel(fileName, GetSaveFlag());

      }
    }
    private void btnOpeExeln_Click(object sender, EventArgs e)
    {
      OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.Filter = "Excel 97-2003 Workbook(*.xls)|*.xls|Excel Workbook(*.xlsx)|*.xlsx";
      openFileDialog.FilterIndex = 1;
      openFileDialog.Multiselect = false;
      if (openFileDialog.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
      {
        string stream = openFileDialog.FileName;
        this.fpSpread1.OpenExcel(stream, GetOpenFlag());
      }
    }
    private ExcelOpenFlags GetOpenFlag()
    {
      ExcelOpenFlags result = ExcelOpenFlags.NoFlagsSet;
      foreach (CheckedListComboBoxItem oItem in lstOpenFlag.Items)
      {
        if (oItem.CheckState == CheckState.Checked && oItem.Tag != null)
          result |= (ExcelOpenFlags)oItem.Tag;
      }
      return result;
    }

    private ExcelSaveFlags GetSaveFlag()
    {
      ExcelSaveFlags result = ExcelSaveFlags.NoFlagsSet;
      foreach (CheckedListComboBoxItem sItem in lstSaveFlag.Items)
      {
        if (sItem.CheckState == CheckState.Checked && sItem.Tag != null)
          result |= (ExcelSaveFlags)sItem.Tag;
      }
      return result;
    }

    private void lstOpenFlag_CheckStateChanged(object sender, ItemCheckEventArgs e)
    {
      if (e.Index == 0)
      {
        if (e.NewValue == CheckState.Checked)
        {
          lstOpenFlag.DisplayText = OPEN_SELECT_ALL;

        }
        else if (e.NewValue == CheckState.Unchecked)
        {
          lstOpenFlag.DisplayText = OPEN_SELECT;
        }
        else
          lstOpenFlag.DisplayText = OPEN_SELECT_SOME;
      }
    }

    private void lstSaveFlag_CheckStateChanged(object sender, ItemCheckEventArgs e)
    {
      if (e.Index == 0)
      {
        if (e.NewValue == CheckState.Checked)
        {
          lstSaveFlag.DisplayText = SAVE_SELECT_ALL;

        }
        else if (e.NewValue == CheckState.Unchecked)
        {
          lstSaveFlag.DisplayText = SAVE_SELECT;
        }
        else
          lstSaveFlag.DisplayText = SAVE_SELECT_SOME;
      }
    }
  }
}
