using FarPoint.PDF;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SpreadWinDemo.PDFExport
{
  public partial class PDFExport : Form
  {
    public PDFExport()
    {
      InitializeComponent();
    }

    private void PDFExport_Load(object sender, EventArgs e)
    {
      FpSpread1.DataSource = Features.Databinding.GetNWINDData("Products");
      FpSpread1.ScrollBarTrackPolicy = FarPoint.Win.Spread.ScrollBarTrackPolicy.Both;

      FpSpread1.Sheets[0].Columns[0].Visible = false;
      FpSpread1.Sheets[0].Columns[1].AllowAutoSort = true;
      FpSpread1.Sheets[0].Columns[8].AllowAutoFilter = true;

      for (int intC = 0; intC < FpSpread1.Sheets[0].ColumnCount; intC++)
        FpSpread1.Sheets[0].Columns[intC].Width = FpSpread1.Sheets[0].GetPreferredColumnWidth(intC);
      FpSpread1.Sheets[0].DefaultStyle.BackColor = Color.AliceBlue;

      Array Values = System.Enum.GetValues(typeof(PdfPermissions));
      foreach (int Value in Values)
      {
        string display = Enum.GetName(typeof(PdfPermissions), Value);
        comboBox1.Items.Add(display);
      }
      comboBox1.SelectedIndex = comboBox1.Items.Count - 1;

    }
    void FpSpread1_PrintMessageBox(object sender, FarPoint.Win.Spread.PrintMessageBoxEventArgs e)
    {
      if (e.BeginPrinting == false)
      {
        if (MessageBox.Show("Would you like to view your PDF File now?", "PDF Export Complete", MessageBoxButtons.YesNo) == DialogResult.Yes)
          System.Diagnostics.Process.Start(saveFileDialog1.FileName);

      }
    }
    private void button1_Click(object sender, EventArgs e)
    {
      string ownerPassword = null, userPassword = null;
      if (checkBox1.Checked)
      {
         ownerPassword = this.textBox1.Text;
         userPassword = this.textBox2.Text;
        if (string.IsNullOrEmpty(ownerPassword) || ownerPassword.Length > 32)
        {
          MessageBox.Show("Please enter valid owner password which is not empty and has length smaller than 32 characters.", "Invalid Owner Password");
          return;
        }
        if (userPassword.Length > 32)
        {
          MessageBox.Show("Please enter valid user password which has length smaller than 32 characters.", "Invalid User Password");
          return;
        }
        if (userPassword.Equals(ownerPassword))
        {
          MessageBox.Show("Owner and User password can not be the same.");
          return;
        }
      }
      saveFileDialog1.Title = "Save Spread to PDF Format";
      saveFileDialog1.Filter = "Pdf File |*.pdf";

      if (saveFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
      {

        FarPoint.Win.Spread.PrintInfo pi = new FarPoint.Win.Spread.PrintInfo();
        pi.PrintToPdf = true;
        pi.PdfWriteMode = FarPoint.Win.Spread.PdfWriteMode.New;
        pi.PdfWriteTo = FarPoint.Win.Spread.PdfWriteTo.File;
        pi.PdfFileName = saveFileDialog1.FileName;
        if (checkBox1.Checked)
        {
          string s = comboBox1.SelectedItem.ToString();
          PdfPermissions permission = PdfPermissions.All;
          try
          {
            permission = (PdfPermissions)Enum.Parse(typeof(PdfPermissions), s);
          }
          catch (System.Exception)
          {
          }
          PdfSecurity security = new PdfSecurity(ownerPassword, userPassword, permission);
          pi.PdfSecurity = security;
        }
        FpSpread1.Sheets[0].PrintInfo = pi;
        FpSpread1.PrintSheet(0);
      }
    }
    private void checkBox1_CheckedChanged(Object sender, EventArgs e)
    {

      CheckBox checkBox = (CheckBox)sender;
      if (checkBox.Checked )
      {
        this.label1.Enabled = true;
        this.textBox1.Enabled = true;
        this.label2.Enabled = true;
        this.textBox2.Enabled = true;
        this.label3.Enabled = true;
        this.comboBox1.Enabled = true;
      }
      else
      {
        this.label1.Enabled = false;
        this.textBox1.Enabled = false;
        this.label2.Enabled = false;
        this.textBox2.Enabled = false;
        this.label3.Enabled = false;
        this.comboBox1.Enabled = false;
      }
    }
  }
}