using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;
using FarPoint.Win.Spread;
using FarPoint.Win.Spread.Chart;
using FarPoint.Win.Spread.DrawingSpace;
using FarPoint.Win.Spread.Model;
using SpreadWinDemo.SparkLineFormEditCustomAxis;

namespace SpreadWinDemo.SparkLineForm
{
  public partial class SparkLineForm : Form
  {
    private ISparklineGroup spGroup = null;
    private ISparkline spLine = null;

    public SparkLineForm(int activeSheetIndex)
      : this()
    {
      FpSpread1.ActiveSheetIndex = activeSheetIndex;
    }
    public SparkLineForm()
    {
      InitializeComponent();
    }

    private void LineSparklineToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ExcelSparklineGroup.ShowSelectDataForm(this, FpSpread1, SparklineType.Line);
    }

    private void ColumnSparklineToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ExcelSparklineGroup.ShowSelectDataForm(this, FpSpread1, SparklineType.Column);
    }

    private void WinLossSparklineToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ExcelSparklineGroup.ShowSelectDataForm(this, FpSpread1, SparklineType.Winloss);
    }

    private void EditGroupDataToolStripMenuItem_Click(object sender, EventArgs e)
    {
      if (spGroup is ExcelSparklineGroup)
        ExcelSparklineGroup.ShowSelectDataForm(this, FpSpread1, spGroup as ExcelSparklineGroup, false);
      else if (spGroup is BulletChartSparklineGroup)
        BulletChartSparklineGroup.ShowSelectDataForm(this, FpSpread1, spGroup);
    }

    private void EditSparklineToolStripMenuItem_Click(object sender, EventArgs e)
    {
      if (spLine is ExcelSparkline)
        ExcelSparklineGroup.ShowSelectDataForm(this, FpSpread1, spLine as ExcelSparkline);
      else if (spLine is BulletChartSparkline)
        BulletChartSparklineGroup.ShowSelectDataForm(this, FpSpread1, spLine, spGroup.Setting);
    }

    private void SwitchRowColumnToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineGroup)spGroup).SwitchRowColumn();
    }

    private void ColorToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ColorDialog dlg = new ColorDialog();

      Color clr = Color.Empty;
      if (spGroup.Setting is ExcelSparklineSetting)
        clr = ((ExcelSparklineSetting)spGroup.Setting).SeriesColor;
      else if (spGroup.Setting is BulletChartSparklineSetting)
        clr = ((BulletChartSparklineSetting)spGroup.Setting).Color;

      dlg.AnyColor = true;
      dlg.Color = clr;
      if (dlg.ShowDialog() == DialogResult.OK)
      {
        if (spGroup.Setting is ExcelSparklineSetting)
          ((ExcelSparklineSetting)spGroup.Setting).SeriesColor = dlg.Color;
        else if (spGroup.Setting is BulletChartSparklineSetting)
          ((BulletChartSparklineSetting)spGroup.Setting).Color = dlg.Color;
        FpSpread1.Invalidate();
      }
    }

    private void HighPointToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).ShowHigh = !((ExcelSparklineSetting)spGroup.Setting).ShowHigh;
      FpSpread1.Invalidate();
    }

    private void LowPointToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).ShowLow = !((ExcelSparklineSetting)spGroup.Setting).ShowLow;
      FpSpread1.Invalidate();
    }

    private void NegativePointToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).ShowNegative = !((ExcelSparklineSetting)spGroup.Setting).ShowNegative;
      FpSpread1.Invalidate();
    }

    private void FirstPointToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).ShowFirst = !((ExcelSparklineSetting)spGroup.Setting).ShowFirst;
      FpSpread1.Invalidate();
    }

    private void LastPointToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).ShowLast = !((ExcelSparklineSetting)spGroup.Setting).ShowLast;
      FpSpread1.Invalidate();
    }

    private void MarkersToolStripMenuItem1_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).ShowMarkers = !((ExcelSparklineSetting)spGroup.Setting).ShowMarkers;
      FpSpread1.Invalidate();
    }

    private void NegativePointsColorToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ColorDialog dlg = new ColorDialog();
      Color clr = ((ExcelSparklineSetting)spGroup.Setting).NegativeColor;
      dlg.AnyColor = true;
      dlg.Color = clr;
      if (dlg.ShowDialog() == DialogResult.OK)
      {
        ((ExcelSparklineSetting)spGroup.Setting).NegativeColor = dlg.Color;
        FpSpread1.Invalidate();
      }
    }

    private void MarkersColorToolStripMenuItem2_Click(object sender, EventArgs e)
    {
      ColorDialog dlg = new ColorDialog();
      Color clr = ((ExcelSparklineSetting)spGroup.Setting).MarkersColor;
      dlg.AnyColor = true;
      dlg.Color = clr;
      if (dlg.ShowDialog() == DialogResult.OK)
      {
        ((ExcelSparklineSetting)spGroup.Setting).MarkersColor = dlg.Color;
        FpSpread1.Invalidate();
      }
    }

    private void HighPointColorToolStripMenuItem1_Click(object sender, EventArgs e)
    {
      ColorDialog dlg = new ColorDialog();
      Color clr = ((ExcelSparklineSetting)spGroup.Setting).HighMarkerColor;
      dlg.AnyColor = true;
      dlg.Color = clr;
      if (dlg.ShowDialog() == DialogResult.OK)
      {
        ((ExcelSparklineSetting)spGroup.Setting).HighMarkerColor = dlg.Color;
        FpSpread1.Invalidate();
      }
    }

    private void LowPointColorToolStripMenuItem1_Click(object sender, EventArgs e)
    {
      ColorDialog dlg = new ColorDialog();
      Color clr = ((ExcelSparklineSetting)spGroup.Setting).LowMarkerColor;
      dlg.AnyColor = true;
      dlg.Color = clr;
      if (dlg.ShowDialog() == DialogResult.OK)
      {
        ((ExcelSparklineSetting)spGroup.Setting).LowMarkerColor = dlg.Color;
        FpSpread1.Invalidate();
      }
    }

    private void FirstPointColorToolStripMenuItem1_Click(object sender, EventArgs e)
    {
      ColorDialog dlg = new ColorDialog();
      Color clr = ((ExcelSparklineSetting)spGroup.Setting).FirstMarkerColor;
      dlg.AnyColor = true;
      dlg.Color = clr;
      if (dlg.ShowDialog() == DialogResult.OK)
      {
        ((ExcelSparklineSetting)spGroup.Setting).FirstMarkerColor = dlg.Color;
        FpSpread1.Invalidate();
      }
    }

    private void LastPointColorToolStripMenuItem1_Click(object sender, EventArgs e)
    {
      ColorDialog dlg = new ColorDialog();
      Color clr = ((ExcelSparklineSetting)spGroup.Setting).LastMarkerColor;
      dlg.AnyColor = true;
      dlg.Color = clr;
      if (dlg.ShowDialog() == DialogResult.OK)
      {
        ((ExcelSparklineSetting)spGroup.Setting).LastMarkerColor = dlg.Color;
        FpSpread1.Invalidate();
      }
    }

    private void AxisGeneralToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).DateAxis = false;
      FpSpread1.Invalidate();
    }

    private void AxisDateToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ExcelSparklineGroup.ShowSelectDataForm(this, FpSpread1, spGroup as ExcelSparklineGroup, true);
      FpSpread1.Invalidate();
    }

    private void ShowAxisToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).DisplayXAxis = !((ExcelSparklineSetting)spGroup.Setting).DisplayXAxis;
      FpSpread1.Invalidate();
    }

    private void PlotDataRightToLeftToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).RightToLeft = !((ExcelSparklineSetting)spGroup.Setting).RightToLeft;
      FpSpread1.Invalidate();
    }

    private void MinimumAutomaticToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).MinAxisType = SparklineAxisMinMax.Individual;
      FpSpread1.Invalidate();
    }

    private void MinimumSameToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).MinAxisType = SparklineAxisMinMax.Group;
      FpSpread1.Invalidate();
    }

    private void MinimumCustomToolStripMenuItem_Click(object sender, EventArgs e)
    {
      try
      {
        EditCustomAxis editAxis = new EditCustomAxis("Specify Custom Axis Minimum", ((ExcelSparklineSetting)spGroup.Setting).ManualMin.ToString());
        if (editAxis.ShowDialog(this) == DialogResult.OK)
        {
          ((ExcelSparklineSetting)spGroup.Setting).ManualMin = Convert.ToDouble(editAxis.textBox1.Text);
          ((ExcelSparklineSetting)spGroup.Setting).MinAxisType = SparklineAxisMinMax.Custom;
          FpSpread1.Invalidate();
        }
      }
      catch (Exception ex)
      {
      }
    }

    private void MaximumAutomaticToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).MaxAxisType = SparklineAxisMinMax.Individual;
      FpSpread1.Invalidate();
    }

    private void MaximumSameToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).MaxAxisType = SparklineAxisMinMax.Group;
      FpSpread1.Invalidate();
    }

    private void MaximumCustomToolStripMenuItem_Click(object sender, EventArgs e)
    {
      try
      {
        EditCustomAxis editAxis = new EditCustomAxis("Specify Custom Axis Maximum", ((ExcelSparklineSetting)spGroup.Setting).ManualMax.ToString());
        if (editAxis.ShowDialog(this) == DialogResult.OK)
        {
          ((ExcelSparklineSetting)spGroup.Setting).ManualMax = Convert.ToDouble(editAxis.textBox1.Text);
          ((ExcelSparklineSetting)spGroup.Setting).MaxAxisType = SparklineAxisMinMax.Custom;
          FpSpread1.Invalidate();
        }
      }
      catch (Exception ex)
      {
      }
    }

    private void GroupSparklinesToolStripMenuItem_Click(object sender, EventArgs e)
    {
      FpSpread1.ActiveSheet.ActiveCell.GroupSparkline(FpSpread1.ActiveSheet.GetSelection(0));
    }

    private void UngroupSparklinesToolStripMenuItem_Click(object sender, EventArgs e)
    {
      FpSpread1.ActiveSheet.UngroupSparkline(FpSpread1.ActiveSheet.GetSelection(0));
    }

    private void ClearSparklinesToolStripMenuItem_Click(object sender, EventArgs e)
    {
      CellRange cellRange = FpSpread1.ActiveSheet.GetSelection(0);
      if (cellRange == null)
        cellRange = new CellRange(FpSpread1.ActiveSheet.ActiveRowIndex, FpSpread1.ActiveSheet.ActiveColumnIndex, 1, 1);
      for (int r = cellRange.Row; r < cellRange.Row + cellRange.RowCount; r++)
        for (int c = cellRange.Column; c < cellRange.Column + cellRange.ColumnCount; c++)
          FpSpread1.ActiveSheet.SparklineContainer.RemoveSparkline(r, c);
    }

    private void SparklineToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
    {
      CellRange cellRange = FpSpread1.ActiveSheet.GetSelection(0);
      if (cellRange == null)
        cellRange = new CellRange(FpSpread1.ActiveSheet.ActiveRowIndex, FpSpread1.ActiveSheet.ActiveColumnIndex, 1, 1);
      FpSpread1.ActiveSheet.SparklineContainer.GetSparkline(FpSpread1.ActiveSheet.ActiveCell.Row.Index, FpSpread1.ActiveSheet.ActiveCell.Column.Index, out spGroup, out spLine);
      if (spLine == null)
      {
        EditToolStripMenuItem.Enabled = false;
        EditSparklineToolStripMenuItem.Enabled = false;
        HiddenEmptyCellsToolStripMenuItem.Enabled = false;
        HiddenEmptyCellsToolStripMenuItem.AllowDrop = false;
        HiddenEmptyCellsToolStripMenuItem.Visible = true;
        SwitchRowColumnToolStripMenuItem.Enabled = false;
        ColorToolStripMenuItem.Enabled = false;
        MarkersToolStripMenuItem.Enabled = false;
        MarkerColorsToolStripMenuItem.Enabled = false;
        AxisToolStripMenuItem.Enabled = false;
        GroupSparklinesToolStripMenuItem.Enabled = false;
        ClearSparklinesToolStripMenuItem.Enabled = false;
      }
      else
      {
        if (spLine is ExcelSparkline)
        {
          EditToolStripMenuItem.Enabled = true;
          EditSparklineToolStripMenuItem.Enabled = true;
          HiddenEmptyCellsToolStripMenuItem.Enabled = true;
          HiddenEmptyCellsToolStripMenuItem.AllowDrop = true;
          HiddenEmptyCellsToolStripMenuItem.Visible = true;
          SwitchRowColumnToolStripMenuItem.Enabled = true;
          ColorToolStripMenuItem.Enabled = true;
          MarkersToolStripMenuItem.Enabled = true;
          MarkerColorsToolStripMenuItem.Enabled = true;
          AxisToolStripMenuItem.Enabled = true;
          GroupSparklinesToolStripMenuItem.Enabled = true;
          ClearSparklinesToolStripMenuItem.Enabled = true;
        }
        else if (spLine is BulletChartSparkline)
        {
          EditToolStripMenuItem.Enabled = true;
          EditSparklineToolStripMenuItem.Enabled = true;
          HiddenEmptyCellsToolStripMenuItem.Enabled = false;
          HiddenEmptyCellsToolStripMenuItem.AllowDrop = false;
          HiddenEmptyCellsToolStripMenuItem.Visible = false;
          SwitchRowColumnToolStripMenuItem.Enabled = false;
          ColorToolStripMenuItem.Enabled = true;
          MarkersToolStripMenuItem.Enabled = false;
          MarkerColorsToolStripMenuItem.Enabled = false;
          AxisToolStripMenuItem.Enabled = false;
          GroupSparklinesToolStripMenuItem.Enabled = false;
          UngroupSparklinesToolStripMenuItem.Enabled = false;
          ClearSparklinesToolStripMenuItem.Enabled = true;
        }
      }
      if (spGroup == null || ((spGroup is BulletChartSparklineGroup) && ((BulletChartSparklineGroup)spGroup).Count == 1) || ((spGroup is ExcelSparklineGroup) && ((ExcelSparklineGroup)spGroup).Count == 1))
      {
        EditGroupDataToolStripMenuItem.Enabled = false;
        UngroupSparklinesToolStripMenuItem.Enabled = false;
      }
      else
      {
        EditGroupDataToolStripMenuItem.Enabled = true;
        if (spGroup is ExcelSparklineGroup)
          UngroupSparklinesToolStripMenuItem.Enabled = true;
      }
      // check selection for other sparklines to group with
      bool found = false;
      if (spGroup != null)
      {
        ISparkline tempLine = null;
        ISparklineGroup tempGroup = null;
        for (int r = cellRange.Row; r < cellRange.Row + cellRange.RowCount; r++)
        {
          for (int c = cellRange.Column; c < cellRange.Column + cellRange.ColumnCount; c++)
          {
            if (r == FpSpread1.ActiveSheet.ActiveRowIndex && c == FpSpread1.ActiveSheet.ActiveColumnIndex)
              continue;
            FpSpread1.ActiveSheet.SparklineContainer.GetSparkline(r, c, out tempGroup, out tempLine);
            if (tempLine != null && !tempGroup.Equals(spGroup))
              found = true;
            break;
          }
          if (found)
            break;
        }
      }
      if (found)
        GroupSparklinesToolStripMenuItem.Enabled = true;
      else
        GroupSparklinesToolStripMenuItem.Enabled = false;
    }

    private void ShowEmptyCellsAsToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
    {
      if (spGroup is ExcelSparklineGroup)
      {
        ExcelSparklineSetting setting = (ExcelSparklineSetting)spGroup.Setting;
        GapsToolStripMenuItem.Checked = false;
        ZeroToolStripMenuItem.Checked = false;
        ConnectToolStripMenuItem.Checked = false;
        if (setting.DisplayEmptyCellsAs == EmptyValueStyle.Gaps)
          GapsToolStripMenuItem.Checked = true;
        else if (setting.DisplayEmptyCellsAs == EmptyValueStyle.Zero)
          ZeroToolStripMenuItem.Checked = true;
        else
          ConnectToolStripMenuItem.Checked = true;
      }
    }

    private void HiddenEmptyCellsToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
    {
      if (spGroup is ExcelSparklineGroup)
        ShowDataInHiddenRowsAndColumnsToolStripMenuItem.Checked = ((ExcelSparklineSetting)spGroup.Setting).DisplayHidden;
    }

    private void GapsToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).DisplayEmptyCellsAs = EmptyValueStyle.Gaps;
      FpSpread1.Invalidate();
    }

    private void ZeroToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).DisplayEmptyCellsAs = EmptyValueStyle.Zero;
      FpSpread1.Invalidate();
    }

    private void ConnectToolStripMenuItem_Click(object sender, EventArgs e)
    {
      ((ExcelSparklineSetting)spGroup.Setting).DisplayEmptyCellsAs = EmptyValueStyle.Connect;
      FpSpread1.Invalidate();
    }

    private void ShowDataInHiddenRowsAndColumnsToolStripMenuItem_Click(object sender, EventArgs e)
    {
      if (spGroup is ExcelSparklineGroup)
      {
        ((ExcelSparklineSetting)spGroup.Setting).DisplayHidden = !((ExcelSparklineSetting)spGroup.Setting).DisplayHidden;
        FpSpread1.Invalidate();
      }
    }

    private void MarkersToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
    {
      HighPointToolStripMenuItem.Checked = ((ExcelSparklineSetting)spGroup.Setting).ShowHigh;
      LowPointToolStripMenuItem.Checked = ((ExcelSparklineSetting)spGroup.Setting).ShowLow;
      NegativePointToolStripMenuItem.Checked = ((ExcelSparklineSetting)spGroup.Setting).ShowNegative;
      FirstPointToolStripMenuItem.Checked = ((ExcelSparklineSetting)spGroup.Setting).ShowFirst;
      LastPointToolStripMenuItem.Checked = ((ExcelSparklineSetting)spGroup.Setting).ShowLast;
      MarkersToolStripMenuItem1.Checked = ((ExcelSparklineSetting)spGroup.Setting).ShowMarkers;
      if (spGroup.Renderer is LineSparklineRenderer)
        MarkersToolStripMenuItem1.Enabled = true;
      else
        MarkersToolStripMenuItem1.Enabled = false;
    }

    private void AxisToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
    {
      if (spGroup is ExcelSparklineGroup)
      {
        AxisGeneralToolStripMenuItem.Checked = !((ExcelSparklineSetting)spGroup.Setting).DateAxis;
        AxisDateToolStripMenuItem.Checked = ((ExcelSparklineSetting)spGroup.Setting).DateAxis;
        ShowAxisToolStripMenuItem.Checked = ((ExcelSparklineSetting)spGroup.Setting).DisplayXAxis;
        PlotDataRightToLeftToolStripMenuItem.Checked = ((ExcelSparklineSetting)spGroup.Setting).RightToLeft;
      }
    }

    private void VerticalAxisMinimumToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
    {
      MinimumAutomaticToolStripMenuItem.Checked = (((ExcelSparklineSetting)spGroup.Setting).MinAxisType == SparklineAxisMinMax.Individual);
      MinimumSameToolStripMenuItem.Checked = (((ExcelSparklineSetting)spGroup.Setting).MinAxisType == SparklineAxisMinMax.Group);
      MinimumCustomToolStripMenuItem.Checked = (((ExcelSparklineSetting)spGroup.Setting).MinAxisType == SparklineAxisMinMax.Custom);
    }

    private void VerticalAxisMaximumToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
    {
      MaximumAutomaticToolStripMenuItem.Checked = (((ExcelSparklineSetting)spGroup.Setting).MaxAxisType == SparklineAxisMinMax.Individual);
      MaximumSameToolStripMenuItem.Checked = (((ExcelSparklineSetting)spGroup.Setting).MaxAxisType == SparklineAxisMinMax.Group);
      MaximumCustomToolStripMenuItem.Checked = (((ExcelSparklineSetting)spGroup.Setting).MaxAxisType == SparklineAxisMinMax.Custom);
    }
    private void V6Features_Load(object sender, EventArgs e)
    {
      FpSpread1.ActiveSheet.SelectionPolicy = SelectionPolicy.MultiRange;
      RectangleShape rectangleShape;
      LineShape arrow;

      rectangleShape = new RectangleShape();
      rectangleShape.CanRenderText = true;
      rectangleShape.Text = "First";
      rectangleShape.Font = new Font("Tahoma", 11.25F);
      rectangleShape.BackColor = SystemColors.Info;
      rectangleShape.ForeColor = SystemColors.InfoText;
      rectangleShape.SetBounds(587, 373, 54, 21);
      FpSpread1_Sheet1.AddShape(rectangleShape);

      rectangleShape = new RectangleShape();
      rectangleShape.CanRenderText = true;
      rectangleShape.Text = "High";
      rectangleShape.Font = new Font("Tahoma", 11.25F);
      rectangleShape.BackColor = SystemColors.Info;
      rectangleShape.ForeColor = SystemColors.InfoText;
      rectangleShape.SetBounds(657, 303, 54, 21);
      FpSpread1_Sheet1.AddShape(rectangleShape);

      rectangleShape = new RectangleShape();
      rectangleShape.CanRenderText = true;
      rectangleShape.Text = "Low";
      rectangleShape.Font = new Font("Tahoma", 11.25F);
      rectangleShape.BackColor = SystemColors.Info;
      rectangleShape.ForeColor = SystemColors.InfoText;
      rectangleShape.SetBounds(676, 387, 54, 21);
      FpSpread1_Sheet1.AddShape(rectangleShape);

      rectangleShape = new RectangleShape();
      rectangleShape.CanRenderText = true;
      rectangleShape.Text = "Negative";
      rectangleShape.Font = new Font("Tahoma", 11.25F);
      rectangleShape.BackColor = SystemColors.Info;
      rectangleShape.ForeColor = SystemColors.InfoText;
      rectangleShape.SetBounds(759, 387, 79, 21);
      FpSpread1_Sheet1.AddShape(rectangleShape);

      rectangleShape = new RectangleShape();
      rectangleShape.CanRenderText = true;
      rectangleShape.Text = "Last";
      rectangleShape.Font = new Font("Tahoma", 11.25F);
      rectangleShape.BackColor = SystemColors.Info;
      rectangleShape.ForeColor = SystemColors.InfoText;
      rectangleShape.SetBounds(775, 304, 54, 21);
      FpSpread1_Sheet1.AddShape(rectangleShape);

      arrow = new LineShape();
      arrow.StartCap = LineCap.ArrowAnchor;
      arrow.EndCap = LineCap.Flat;
      arrow.SetBounds(640, 359, 18, 15);
      arrow.FlipHorizontal = true;
      FpSpread1_Sheet1.AddShape(arrow);

      arrow = new LineShape();
      arrow.StartCap = LineCap.Flat;
      arrow.EndCap = LineCap.ArrowAnchor;
      arrow.SetBounds(683, 325, 1, 20);
      FpSpread1_Sheet1.AddShape(arrow);

      arrow = new LineShape();
      arrow.StartCap = LineCap.ArrowAnchor;
      arrow.EndCap = LineCap.Flat;
      arrow.SetBounds(704, 367, 1, 20);
      FpSpread1_Sheet1.AddShape(arrow);

      arrow = new LineShape();
      arrow.StartCap = LineCap.ArrowAnchor;
      arrow.EndCap = LineCap.Flat;
      arrow.SetBounds(729, 363, 31, 24);
      FpSpread1_Sheet1.AddShape(arrow);

      arrow = new LineShape();
      arrow.EndCap = LineCap.ArrowAnchor;
      arrow.StartCap = LineCap.Flat;
      arrow.SetBounds(752, 325, 24, 24);
      arrow.FlipHorizontal = true;
      FpSpread1_Sheet1.AddShape(arrow);

      int count = 42;
      int i = 36;
      do
      {
        this.BindSparklineInRow(i);
        i++;
      }
      while (i <= count);
    }
    private void BindSparklineInRow(int row)
    {
      BulletChartSparklineSetting bulletSetting = new BulletChartSparklineSetting(Color.Gray, false, 10.0);
      BulletChartSparklineGroup bulletGroup = new BulletChartSparklineGroup();
      bulletGroup.Setting = bulletSetting;
      SheetCellRange a1 = new SheetCellRange(this.FpSpread1.Sheets[0], row, 6, 1, 1);
      SheetCellRange a2 = new SheetCellRange(this.FpSpread1.Sheets[0], row, 7, 1, 1);
      SheetCellRange a3 = new SheetCellRange(this.FpSpread1.Sheets[0], row, 8, 1, 1);
      SheetCellRange a4 = new SheetCellRange(this.FpSpread1.Sheets[0], row, 9, 1, 1);
      SheetCellRange a5 = new SheetCellRange(this.FpSpread1.Sheets[0], row, 10, 1, 1);
      SheetCellRange a6 = new SheetCellRange(this.FpSpread1.Sheets[0], row, 11, 1, 1);
      BulletChartSparkline bulletSparkline = new BulletChartSparkline(this.FpSpread1.Sheets[0], row, 12, a1, a2, a3, a4, a5, a6);
      if (_bullGroup == null)
      {
        _bullGroup = new BulletChartSparklineGroup();
        _bullGroup.Setting = bulletSetting;
        _bullGroup.Add(bulletSparkline);
        this.FpSpread1.ActiveSheet.SparklineContainer.Add(_bullGroup);
      }
      else
      {
        _bullGroup.Add(bulletSparkline);
      }
    }
    private BulletChartSparklineGroup _bullGroup = null;
    private void bulletChartSparklineToolStripMenuItem_Click(object sender, EventArgs e)
    {
      BulletChartSparklineGroup.ShowSelectDataForm(this, FpSpread1, new BulletChartSparklineSetting(Color.Gray, false, 10.0));
    }
  }
}