﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.CompilerServices;
using FarPoint.Win.Spread;
using System.Diagnostics;
using System.Reflection;
using FarPoint.Win.Spread.Chart;
using FarPoint.Win.Spread.Model;
using System.Collections;

namespace SpreadWinDemo.SparkLineForm
{
  public partial class SparklineSelectDataForm : ModelessForm
  {
    // Fields

    private ISparklineGroup _group;

    private Type _groupType;

    private FormMode _mode;

    private ISparklineSetting _setting;
    private ISparkline _sparkline;
    private Type _sparklineType;
    private FpSpread _spread;
    private List<MemberInfo> dataMembers;
    private ModelessTextBox MyActivatingTextBox;

    public SparklineSelectDataForm()
    {
            base.FormClosed += new FormClosedEventHandler(this.BulletChartSparklineSelectData_FormClosed);
            List<WeakReference> VBt_refL0 = __ENCList;
            lock (VBt_refL0)
            {
                __ENCList.Add(new WeakReference(this));
            }
            this._mode = FormMode.Create;
            this.dataMembers = new List<MemberInfo>();
            this.InitializeComponent();
            base.Controls["CompactPanel"].Controls["ChartDataIndicatorButton"].Dock = DockStyle.Right;
            base.Controls["CompactPanel"].Controls["SelectCellRangeTextBoxBase"].Anchor = AnchorStyles.Right | AnchorStyles.Left | AnchorStyles.Bottom | AnchorStyles.Top;
        }
    // Methods
    /// <summary>
    /// Initializes a new instance of the create sparkline form<see cref="SparklineSelectDataForm"/> class.
    /// </summary>
    /// <param name="spread">The spread.</param>
    /// <param name="sparklineType">Type of the sparkline.</param>
    /// <param name="groupType">Type of the group.</param>
    /// <param name="setting">The setting.</param>
    public SparklineSelectDataForm(FpSpread spread, Type sparklineType, Type groupType, ISparklineSetting setting)
      : base(spread)
    {
      base.FormClosed += new FormClosedEventHandler(this.BulletChartSparklineSelectData_FormClosed);
      List<WeakReference> VBt_refL0 = __ENCList;
      lock (VBt_refL0)
      {
        __ENCList.Add(new WeakReference(this));
      }
      this._mode = FormMode.Create;
      this.dataMembers = new List<MemberInfo>();
      this._spread = spread;
      this.InitializeComponent();
      base.Controls["CompactPanel"].Controls["ChartDataIndicatorButton"].Dock = DockStyle.Right;
      base.Controls["CompactPanel"].Controls["SelectCellRangeTextBoxBase"].Anchor = AnchorStyles.Right | AnchorStyles.Left | AnchorStyles.Bottom | AnchorStyles.Top;
      this.SparklineType = sparklineType;
      this._groupType = groupType;
      this._setting = setting;
    }

    /// <summary>
    /// Initializes a new instance of the edit single sparkline form<see cref="SparklineSelectDataForm"/> class.
    /// </summary>
    /// <param name="spread">The spread.</param>
    /// <param name="sparkline">The sparkline.</param>
    /// <param name="sparklineType">Type of the sparkline.</param>
    /// <param name="groupType">Type of the group.</param>
    /// <param name="setting">The setting.</param>
    public SparklineSelectDataForm(FpSpread spread, ISparkline sparkline, Type sparklineType, Type groupType, ISparklineSetting setting)
      : this(spread, sparklineType, groupType, setting)
    {
      this._sparkline = sparkline;
      if (this._sparkline != null)
      {
        this._mode = FormMode.EditSingle;
        this.GroupBox2.Visible = false;
        this.Height -= this.GroupBox2.Height;
        ISparklineGroup g = (ISparklineGroup)this._groupType.InvokeMember(this._groupType.Name, BindingFlags.CreateInstance, null, null, null);
        g.Add(this._sparkline);
        this.GetSparklineData(g);
      }
    }

    /// <summary>
    /// Initializes a new instance of the sparkline group form<see cref="SparklineSelectDataForm"/> class.
    /// </summary>
    /// <param name="spread">The spread.</param>
    /// <param name="group">The group.</param>
    /// <param name="sparklineType">Type of the sparkline.</param>
    /// <param name="groupType">Type of the group.</param>
    /// <param name="setting">The setting.</param>
    public SparklineSelectDataForm(FpSpread spread, ISparklineGroup group, Type sparklineType, Type groupType, ISparklineSetting setting)
      : this(spread, sparklineType, groupType, setting)
    {
      this._group = group;
      if ((this._group != null) && (this._group.Count > 0))
      {
        this._mode = FormMode.EditGroup;
        this.GetSparklineData(this._group);
      }
    }

    protected override void ActiveSheetView(string formula, SheetView defaultSheetView)
    {
      if (defaultSheetView == null)
      {
        defaultSheetView = this._spread.ActiveSheet;
      }
      base.ActiveSheetView(formula, defaultSheetView);
    }

    private void BulletChartSparklineSelectData_FormClosed(object sender, FormClosedEventArgs e)
    {
      this._spread.ActiveSheet.ClearSelection();
    }

    private void Cancel_Click(object sender, EventArgs e)
    {
      this.Close();
    }

    protected override void SetVisibleCore(bool value)
    {
      if (value)
        base.SetVisibleCore(value);
    }
    private void FillDataText(int index, int count, List<SheetCellRange> list)
    {
      ModelessTextBox modelessTextBox = (ModelessTextBox)this.GroupBox1.Controls[(index * 2) - 1];
      modelessTextBox.Text = modelessTextBox.Text + "=";
      int loopCount = (index * count) - 1;
      for (int i = (index - 1) * count; i <= loopCount; i++)
      {
        modelessTextBox = (ModelessTextBox)this.GroupBox1.Controls[(index * 2) - 1];
        modelessTextBox.Text = modelessTextBox.Text + this.GetFormula(this._spread.ActiveSheet, list[i]).Trim(new char[] { '=' });
        if (i < ((index * count) - 1))
        {
          modelessTextBox = (ModelessTextBox)this.GroupBox1.Controls[(index * 2) - 1];
          modelessTextBox.Text = modelessTextBox.Text + ",";
        }
      }
    }

    private void GetSparklineData(ISparklineGroup gr)
    {
      string formula = string.Empty;
      int groupCount = gr.Count - 1;
      for (int i = 0; i <= groupCount; i++)
      {
        ModelessTextBox modelessTextBox;
        ISparkline s = gr[i];
        int memberCount = this.dataMembers.Count - 1;
        for (int j = 0; j <= memberCount; j++)
        {
          object o = RuntimeHelpers.GetObjectValue(this._sparklineType.InvokeMember(this.dataMembers[j].Name, BindingFlags.GetProperty, null, s, null));
          if (o.GetType() == typeof(SheetCellRange))
          {
            SheetCellRange sc = (SheetCellRange)o;
            formula = this.GetFormula(sc.SheetView, new CellRange(sc.Row, sc.Column, sc.RowCount, sc.ColumnCount)).Trim(new char[] { '=' });
          }
          else if (o.GetType() == typeof(CellRangeSegmentData))
          {
            CellRangeSegmentData sc = (CellRangeSegmentData)o;
            formula = this.GetFormula(sc.SheetView, new CellRange(sc.Row, sc.Column, sc.RowCount, sc.ColumnCount)).Trim(new char[] { '=' });
          }
          if (i == 0)
          {
            modelessTextBox = (ModelessTextBox)this.GroupBox1.Controls[(2 * (j + 1)) - 1];
            modelessTextBox.Text = modelessTextBox.Text + "=";
          }
          modelessTextBox = (ModelessTextBox)this.GroupBox1.Controls[(2 * (j + 1)) - 1];
          modelessTextBox.Text = modelessTextBox.Text + formula;
          if (i < (gr.Count - 1))
          {
            modelessTextBox = (ModelessTextBox)this.GroupBox1.Controls[(2 * (j + 1)) - 1];
            modelessTextBox.Text = modelessTextBox.Text + ",";
          }
        }
        if (i == 0)
        {
          modelessTextBox = this.LocationTextBox;
          modelessTextBox.Text = modelessTextBox.Text + "=";
        }
        modelessTextBox = this.LocationTextBox;
        modelessTextBox.Text = modelessTextBox.Text + this.GetFormula(this._spread.ActiveSheet, new CellRange(s.Row, s.Column, 1, 1)).Trim(new char[] { '=' });
        if (i < (gr.Count - 1))
        {
          modelessTextBox = this.LocationTextBox;
          modelessTextBox.Text = modelessTextBox.Text + ",";
        }
      }
    }
    private void OK_Click(object sender, EventArgs e)
    {
      try
      {
        this.ProcessSelectDataSources();
      }
      catch
      {
        MessageBox.Show("Input is not valid");
      }
    }

    protected override void OnLoad(EventArgs e)
    {
      this.GroupBox1.AutoSize = true;
      if (this._mode == FormMode.Create)
      {
        List<SheetCellRange> list = new List<SheetCellRange>();
        SheetCellRange[] ranges = base.GetSheetCellRangeArray(this._spread.ActiveSheet, this._spread.ActiveSheet.GetSelections());
        for (int i = 0; i < ranges.Length; i++)
        {
          list.Add(ranges[i]);
        }
        this.SeparateCellRangeToCell(list);
        if ((list.Count == 1) | (list.Count == 0))
        {
          this.ActiveControl = this.GroupBox1.Controls[1];
          this.LocationTextBox.Text = this.GetFormula(this._spread.ActiveSheet, new CellRange(this._spread.ActiveSheet.ActiveCell.Row.Index, this._spread.ActiveSheet.ActiveCell.Column.Index, 1, 1));
        }
        else if ((list.Count % this.dataMembers.Count) == 0)
        {
          int count = (int)Math.Round((double)(((double)list.Count) / ((double)this.dataMembers.Count)));
          int dataMemberCount = this.dataMembers.Count;
          for (int i = 1; i <= dataMemberCount; i++)
          {
            this.FillDataText(i, count, list);
          }
          this.ActiveControl = this.LocationTextBox;
        }
      }
      else
      {
        this.ActiveControl = this.LocationTextBox;
      }
    }

    protected override void OnSelectionChanged(CellRange cellRange, CellRange[] allSelectedCellRanges)
    {
      base.OnSelectionChanged(cellRange, allSelectedCellRanges);
      this.UpdateSelectionCellRange(this._spread.ActiveSheet, allSelectedCellRanges);
      if (!this.IsInCompactMode)
      {
        base.AutoCompact = true;
      }
    }

    private void ProcessSelectDataSources()
    {
      if (this.dataMembers.Count <= 0)
      {
        throw new Exception("Input is not valid");
      }
      List<SheetCellRange> cellRangeLocation = new List<SheetCellRange>();
      if (this._mode != FormMode.EditSingle)
      {
        cellRangeLocation = base.ParseCellRanges(this.LocationTextBox.Text, true);
        this.SeparateCellRangeToCell(cellRangeLocation);
      }
      List<List<SheetCellRange>> dataList = new List<List<SheetCellRange>>();
      int dataMemberCount = this.dataMembers.Count - 1;
      for (int i = 0; i <= dataMemberCount; i++)
      {
        List<SheetCellRange> listCellRange = base.ParseCellRanges(((ModelessTextBox)this.GroupBox1.Controls[((i + 1) * 2) - 1]).Text, true);
        if (((CellRangeSegmentDataSupportAttribute)this.dataMembers[i].GetCustomAttributes(typeof(CellRangeSegmentDataSupportAttribute), true)[0]).MultiCell)
        {
          this.SeparateCellRangeToOneLineCellRange(listCellRange, cellRangeLocation.Count);
        }
        else
        {
          this.SeparateCellRangeToCell(listCellRange);
        }
        dataList.Add(listCellRange);
      }
      ISparklineGroup g = (ISparklineGroup)this._groupType.InvokeMember(this._groupType.Name, BindingFlags.CreateInstance, null, null, null);
      g.Setting = this._setting;
      int dataListCount = dataList[0].Count - 1;
      for (int i = 0; i <= dataListCount; i++)
      {
        ISparkline s = (ISparkline)this._sparklineType.InvokeMember(this._sparklineType.Name, BindingFlags.CreateInstance, null, null, null);
        for (int j = 0; j <= dataMemberCount; j++)
        {
          PropertyInfo pi = this._sparklineType.GetProperty(this.dataMembers[j].Name);
          string propertyName = pi.PropertyType.Name;
          if (propertyName == typeof(SheetCellRange).Name)
          {
            this._sparklineType.InvokeMember(this.dataMembers[j].Name, BindingFlags.SetProperty, null, s, new object[] { dataList[j][i] });
          }
          else if ((propertyName == typeof(ISegmentData).Name) || (propertyName == typeof(CellRangeSegmentData).Name))
          {
            this._sparklineType.InvokeMember(this.dataMembers[j].Name, BindingFlags.SetProperty, null, s, new object[] { this.ToCellRangeSegmentData(dataList[j][i]) });
          }
        }
        if (this._mode != FormMode.EditSingle)
        {
          s.Row = cellRangeLocation[i].Row;
          s.Column = cellRangeLocation[i].Column;
        }
        else
        {
          s.Row = this._sparkline.Row;
          s.Column = this._sparkline.Column;
        }
        g.Add(s);
      }
      if (this._mode == FormMode.Create)
      {
        this._spread.ActiveSheet.SparklineContainer.Add(g);
      }
      else if (this._mode == FormMode.EditGroup)
      {
        this._spread.ActiveSheet.SparklineContainer.Remove(_group);
        this._spread.ActiveSheet.SparklineContainer.Add(g);
      }
      else if (this._mode == FormMode.EditSingle)
      {
        if (g.Count != 1)
        {
          throw new Exception("Input is not valid");
        }
        ISparkline spl = g[0];
        this._sparkline.Row = spl.Row;
        this._sparkline.Column = spl.Column;
        int count = this.dataMembers.Count - 1;
        for (int j = 0; j <= count; j++)
        {
          object o = RuntimeHelpers.GetObjectValue(this._sparklineType.InvokeMember(this.dataMembers[j].Name, BindingFlags.GetProperty, null, spl, null));
          this._sparklineType.InvokeMember(this.dataMembers[j].Name, BindingFlags.SetProperty, null, this._sparkline, new object[] { RuntimeHelpers.GetObjectValue(o) });
        }
      }
      this.Close();
    }

    private void SeparateCellRangeToCell(List<SheetCellRange> ranges)
    {
      List<SheetCellRange> list = new List<SheetCellRange>();
      int rangesCount = ranges.Count - 1;
      for (int i = 0; i <= rangesCount; i++)
      {
        SheetCellRange cr = ranges[i];
        if ((cr.ColumnCount > 1) && (cr.RowCount > 1))
        {
          int crColumnCount = cr.ColumnCount - 1;
          for (int j = 0; j <= crColumnCount; j++)
          {
            int VBt_i4L2 = cr.RowCount - 1;
            for (int k = 0; k <= VBt_i4L2; k++)
            {
              list.Add(new SheetCellRange(cr.SheetView, cr.Row + k, cr.Column + j, 1, 1));
            }
          }
        }
        else if (cr.RowCount > 1)
        {
          int crRowCount = cr.RowCount - 1;
          for (int j = 0; j <= crRowCount; j++)
          {
            list.Add(new SheetCellRange(cr.SheetView, cr.Row + j, cr.Column, 1, 1));
          }
        }
        else if (cr.ColumnCount > 1)
        {
          int crColumnCount = cr.ColumnCount - 1;
          for (int j = 0; j <= crColumnCount; j++)
          {
            list.Add(new SheetCellRange(cr.SheetView, cr.Row, cr.Column + j, 1, 1));
          }
        }
        else if ((cr.ColumnCount == 1) && (cr.RowCount == 1))
        {
          list.Add(cr);
        }
      }
      ranges.Clear();
      ranges.AddRange(list);
      list.Clear();
    }

    private void SeparateCellRangeToOneLineCellRange(List<SheetCellRange> ranges, int number)
    {
      List<SheetCellRange> list = new List<SheetCellRange>();
      int rangesCount = ranges.Count - 1;
      for (int i = 0; i <= rangesCount; i++)
      {
        SheetCellRange cr = ranges[i];
        if ((cr.ColumnCount > 1) && (cr.RowCount > 1))
        {
          if (cr.ColumnCount == number)
          {
            int columnCount = cr.ColumnCount - 1;
            for (int j = 0; j <= columnCount; j++)
            {
              list.Add(new SheetCellRange(cr.SheetView, cr.Row, cr.Column + j, cr.RowCount, 1));
            }
          }
          else
          {
            if (cr.RowCount != number)
            {
              throw new Exception("Invalid input");
            }
            int rowCount = cr.RowCount - 1;
            for (int j = 0; j <= rowCount; j++)
            {
              list.Add(new SheetCellRange(cr.SheetView, cr.Row + j, cr.Column, 1, cr.ColumnCount));
            }
          }
        }
        else
        {
          list.Add(cr);
        }
      }
      ranges.Clear();
      ranges.AddRange(list);
      list.Clear();
    }

    private void TextBox_Enter(object sender, EventArgs e)
    {
      if (sender.GetType() == typeof(ModelessTextBox))
      {
        this.MyActivatingTextBox = (ModelessTextBox)sender;
        base.ActivatingTextBox = this.MyActivatingTextBox.TextBox;
      }
    }

    private void TextBox_Leave(object sender, EventArgs e)
    {
      this._spread.ActiveSheet.ClearSelection();
    }

    private void TextBox_SelectButtonClick(object sender, EventArgs e)
    {
      this.ActivatingTextBox = ((ModelessTextBox)sender).TextBox;
      this.TransformToCompactMode();
      base.AutoCompact = false;
    }

    private CellRange ToCellRange(CellRangeSegmentData range)
    {
      return new CellRange(range.Row, range.Column, range.RowCount, range.ColumnCount);
    }

    private CellRangeSegmentData ToCellRangeSegmentData(SheetCellRange cellRange)
    {
      return new CellRangeSegmentData(cellRange.SheetView, cellRange.Row, cellRange.Column, cellRange.RowCount, cellRange.ColumnCount, new ChartDataSetting());
    }

    protected override void TransformToCompactMode()
    {
      base.TransformToCompactMode();
      this.OnActivatingTextBoxEnter();
    }

    protected override void UpdateSelectionCellRange(SheetView sheetView, CellRange[] allSelectedCellRanges)
    {
      SheetCellRange[] cellRanges = base.GetSheetCellRangeArray(this._spread.ActiveSheet, allSelectedCellRanges);
      base.ActivatingTextBox.Text = this.GetFormula(cellRanges, false);
      base.ActivatingTextBox.SelectionStart = this.ActivatingTextBox.Text.Length;
      this.MyActivatingTextBox.Text = base.ActivatingTextBox.Text;
      this.MyActivatingTextBox.TextBox.SelectionStart = base.ActivatingTextBox.SelectionStart;
    }
    private Type SparklineType
    {
      set
      {
        if (this._sparklineType != value)
        {
          IEnumerator controls = null;
          this._sparklineType = value;
          foreach (MemberInfo m in this._sparklineType.GetMembers())
          {
            if (m.GetCustomAttributes(typeof(CellRangeSegmentDataSupportAttribute), true).Length > 0)
            {
              this.dataMembers.Add(m);
            }
          }
          try
          {
            controls = this.GroupBox1.Controls.GetEnumerator();
            while (controls.MoveNext())
            {
              Control c = (Control)controls.Current;
              if (c.GetType() == typeof(ModelessTextBox))
              {
                c.Enter -= new EventHandler(this.TextBox_Enter);
                ((ModelessTextBox)c).SelectButtonClick -= new EventHandler(this.TextBox_SelectButtonClick);
                c.Leave -= new EventHandler(this.TextBox_Leave);
              }
            }
          }
          finally
          {
            if (controls is IDisposable)
            {
              (controls as IDisposable).Dispose();
            }
          }
          Point loc = new Point(this.Label7.Left, 20);
          Point loc1 = new Point(this.LocationTextBox.Left, 20);
          this.GroupBox1.Controls.Clear();
          this.GroupBox1.Height = 20;
          int dataMemberCount = this.dataMembers.Count - 1;
          for (int i = 0; i <= dataMemberCount; i++)
          {
            CellRangeSegmentDataSupportAttribute att = (CellRangeSegmentDataSupportAttribute)this.dataMembers[i].GetCustomAttributes(typeof(CellRangeSegmentDataSupportAttribute), true)[0];
            Label l = new Label();
            l.AutoSize = true;
            l.Text = att.Text;
            l.Location = loc;
            ModelessTextBox tb = new ModelessTextBox();
            tb.Location = loc1;
            tb.Width = this.LocationTextBox.Width;
            tb.Enter += new EventHandler(this.TextBox_Enter);
            tb.SelectButtonClick += new EventHandler(this.TextBox_SelectButtonClick);
            tb.Leave += new EventHandler(this.TextBox_Leave);
            this.GroupBox1.Controls.Add(l);
            this.GroupBox1.Controls.Add(tb);
            loc.Y += 30;
            loc1.Y += 30;
          }
          int h = this.dataMembers.Count * (30 + this.LocationTextBox.Height);
          this.Size = new Size(this.GroupBox2.Width + 40, this.GroupBox2.Height + h);
        }
      }
    }

    // Nested Types
    //Form mode
    private enum FormMode
    {
      Create,
      EditGroup,
      EditSingle
    }
  }


}
