﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using FarPoint.Win.Spread;
using FarPoint.Win.Spread.Chart;

namespace SpreadWinDemo.SpreadChartDemo
{
  public partial class SpreadChartDemo : Form
  {
    private SpreadChart _chart = null;
    public SpreadChartDemo()
    {
      InitializeComponent();
      this.fpSpread1.OnActiveChartChanged += new MySpread.ActiveShapeChanged(fpSpread1_OnActiveChartChanged);
      this.groupBox1.Enabled = false;
      this.groupBox2.Enabled = false;
    }
    void fpSpread1_OnActiveChartChanged(object sender, object obj)
    {
      _chart = obj as SpreadChart;
      if (_chart != null)
      {
        this.groupBox1.Enabled = true;
        this.groupBox2.Enabled = true;
        if (this._chart != null && this._chart.Model.LegendAreas != null && this._chart.Model.LegendAreas.Count > 0)
        {
          this.checkBox1.Checked = !this._chart.Model.LegendAreas[0].Overlay;
          if (this._chart.Model.LegendAreas[0].Vertical)
          {
            this.radioButton7.Checked = true;
          }
          else
            this.radioButton6.Checked = true;
          switch (this._chart.Model.LegendAreas[0].LegendPos)
          {
            case FarPoint.Win.Chart.LegendPos.Top:
              this.radioButton1.Checked = true;
              break;
            case FarPoint.Win.Chart.LegendPos.Bottom:
              this.radioButton2.Checked = true;
              break;
            case FarPoint.Win.Chart.LegendPos.Left:
              this.radioButton3.Checked = true;
              break;
            case FarPoint.Win.Chart.LegendPos.Right:
              this.radioButton4.Checked = true;
              break;
            case FarPoint.Win.Chart.LegendPos.TopRight:
              this.radioButton5.Checked = true;
              break;
          }
        }
      }
      else
      {
        this.groupBox1.Enabled = false;
        this.groupBox2.Enabled = false;
      }
    }
    private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
    {
      this.Cursor = Cursors.WaitCursor;
      this.LoadSpread(e.Node.Text);
      this.Cursor = Cursors.Default;
    }
    private void LoadSpread(string categoryName)
    {
      try
      {
        string defaultCommonPath = System.IO.Path.Combine(Application.StartupPath, "..\\Common\\");// default XML path
        string commonPath = defaultCommonPath;
        string regSampleVersion = "v" + GCVersionUtil.ProductVersion;
        Microsoft.Win32.RegistryKey commonKey = Microsoft.Win32.Registry.ClassesRoot.OpenSubKey("FarPoint.Win.Spread\\Common7");
        if (commonKey != null)
        {
          object commonPathObj = commonKey.GetValue(regSampleVersion, defaultCommonPath);
          commonPath = commonPathObj is string ? (string)commonPathObj : defaultCommonPath;
        }        
        string installedPath = System.IO.Path.Combine(commonPath, "..\\");
        string chartXmlPath = "Windows Forms\\" + regSampleVersion + "\\ChartXml";
        string xmlFile = System.IO.Path.Combine(System.IO.Path.Combine(installedPath, chartXmlPath), categoryName + ".xml");
        if (!System.IO.File.Exists(xmlFile))
          xmlFile = Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles) + System.IO.Path.Combine(GCVersionUtil.SPREAD_INSTALLATION_FOLDER, chartXmlPath, categoryName + ".xml");
        this.fpSpread1.Open(xmlFile);
      }
      catch
      {
        MessageBox.Show("Can't load data file", "Spread chart demo");
      }
    }

    private void checkBox1_CheckedChanged(object sender, EventArgs e)
    {
      if (this._chart != null && this._chart.Model.LegendAreas != null && this._chart.Model.LegendAreas.Count > 0)
      {
        this._chart.Model.LegendAreas[0].Overlay = !this.checkBox1.Checked;
      }
    }

    private void radioButton1_CheckedChanged(object sender, EventArgs e)
    {
      if (this._chart != null && this._chart.Model.LegendAreas != null && this._chart.Model.LegendAreas.Count > 0)
      {
        this._chart.Model.LegendAreas[0].LegendPos = FarPoint.Win.Chart.LegendPos.Top;
      }
    }

    private void radioButton2_CheckedChanged(object sender, EventArgs e)
    {
      if (this._chart != null && this._chart.Model.LegendAreas != null && this._chart.Model.LegendAreas.Count > 0)
      {
        this._chart.Model.LegendAreas[0].LegendPos = FarPoint.Win.Chart.LegendPos.Bottom;
      }
    }

    private void radioButton3_CheckedChanged(object sender, EventArgs e)
    {
      if (this._chart != null && this._chart.Model.LegendAreas != null && this._chart.Model.LegendAreas.Count > 0)
      {
        this._chart.Model.LegendAreas[0].LegendPos = FarPoint.Win.Chart.LegendPos.Left;
      }
    }

    private void radioButton4_CheckedChanged(object sender, EventArgs e)
    {
      if (this._chart != null && this._chart.Model.LegendAreas != null && this._chart.Model.LegendAreas.Count > 0)
      {
        this._chart.Model.LegendAreas[0].LegendPos = FarPoint.Win.Chart.LegendPos.Right;
      }
    }

    private void radioButton5_CheckedChanged(object sender, EventArgs e)
    {
      if (this._chart != null && this._chart.Model.LegendAreas != null && this._chart.Model.LegendAreas.Count > 0)
      {
        this._chart.Model.LegendAreas[0].LegendPos = FarPoint.Win.Chart.LegendPos.TopRight;
      }
    }

    private void radioButton6_CheckedChanged(object sender, EventArgs e)
    {
      if (this._chart != null && this._chart.Model.LegendAreas != null && this._chart.Model.LegendAreas.Count > 0)
      {
        this._chart.Model.LegendAreas[0].Vertical = false;
      }
    }

    private void radioButton7_CheckedChanged(object sender, EventArgs e)
    {
      if (this._chart != null && this._chart.Model.LegendAreas != null && this._chart.Model.LegendAreas.Count > 0)
      {
        this._chart.Model.LegendAreas[0].Vertical = true;
      }
    }

  }
  /// <summary>
  /// 
  /// </summary>
  class MySpread : FpSpread
  {
    public delegate void ActiveShapeChanged(object sender, Object obj);
    public event ActiveShapeChanged OnActiveChartChanged;
    public override FarPoint.Win.ElementWindowless ActiveWindowlessObject
    {
      get
      {
        return base.ActiveWindowlessObject;
      }
      set
      {
        base.ActiveWindowlessObject = value;
        ChartChanged(this, value);
      }
    }

    private void ChartChanged(MySpread mySpreadWrapper, FarPoint.Win.ElementWindowless value)
    {
      if (OnActiveChartChanged != null)
        OnActiveChartChanged(this, value);
    }
  }
}
