using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using FarPoint.Win.Spread;
using System.Collections;

namespace SpreadWinDemo.StatusBar
{
  public partial class StatusBar : Form
  {
    public StatusBar()
    {
      InitializeComponent();
    }

    private void Form1_Load(object sender, EventArgs e)
    {
      fpSpread1.SuspendLayout();
      fpSpread1.ActiveSheet.RowCount = 100;
      fpSpread1.ActiveSheet.ColumnCount = 30;

      fpSpread1.ResumeLayout();

      lbSpreadSkins.Items.AddRange(DefaultSpreadSkins.Skins);
      lbSpreadSkins.SelectedItem = fpSpread1.Skin;

      fpSpread1.StatusBarVisible = true;
      chkBoxVisibleStatusBar.Checked = true;
      pgSkin.SelectedObject = fpSpread1.StatusBar;

      FpSpread spreadtmp = new FpSpread();
      spreadtmp.Sheets.Count = 1;
      spreadtmp.ActiveSheet.SetActiveCell(10, 10);
      spreadtmp.Size = new System.Drawing.Size(150, 100);
      foreach (SpreadSkin skin in DefaultSpreadSkins.Skins)
      {
        Bitmap image = new Bitmap(150, 100);
        skin.Apply(spreadtmp);
        spreadtmp.DrawToBitmap(image, new Rectangle(new Point(0, 0), spreadtmp.Size));
        cachedSpreadSkinImages.Add(skin.Name, image);
      }
      spreadtmp.Dispose();

    }

    private void listBox1_SelectedIndexChanged(object sender, EventArgs e)
    {
      ListBox lst = sender as ListBox;
      SpreadSkin skin = new SpreadSkin(lst.SelectedItem as SpreadSkin);
      skin.Apply(fpSpread1);
      lst.Invalidate();
    }

    Hashtable cachedSpreadSkinImages = new Hashtable();
    private void listBox1_DrawItem(object sender, DrawItemEventArgs e)
    {
      ListBox lst = sender as ListBox;
      SpreadSkin skin = lst.Items[e.Index] as SpreadSkin;
      Brush br = null;
      if (e.Index == lst.SelectedIndex)
      {
        br = new SolidBrush(Color.FromArgb(159, 213, 183));
        e.Graphics.FillRectangle(br, e.Bounds);
        br.Dispose();
      }
      else if (e.Index == hightIndex)
      {
        br = new SolidBrush(Color.FromArgb(211, 240, 224));
        e.Graphics.FillRectangle(br, e.Bounds);
        br.Dispose();
      }
      Image sampleImg = cachedSpreadSkinImages[skin.Name] as Image;
      if (sampleImg == null)
        return;
      Rectangle imgBound = new Rectangle(e.Bounds.Location, new Size(75, 50));
      imgBound.Inflate(-1, -1);
      e.Graphics.FillRectangle(SystemBrushes.ActiveBorder, imgBound);
      imgBound.Inflate(-1, -1);
      e.Graphics.DrawImage(sampleImg, imgBound);
      StringFormat sf = new StringFormat();
      sf.LineAlignment = StringAlignment.Center;
      Rectangle textBound = new Rectangle(imgBound.Right + 2, e.Bounds.Y, e.Bounds.Right - imgBound.Right - 2, e.Bounds.Height);
      e.Graphics.DrawString(skin.Name, e.Font, SystemBrushes.ControlText, textBound, sf);
    }

    int hightIndex = -1;
    private void lbSpreadSkins_MouseMove(object sender, MouseEventArgs e)
    {
      ListBox list = ((ListBox)sender);
      int idx = list.IndexFromPoint(e.Location);
      if (idx != hightIndex)
      {
        if (hightIndex != -1)
          list.Invalidate(list.GetItemRectangle(hightIndex));
        if (idx != -1)
          list.Invalidate(list.GetItemRectangle(idx));
        hightIndex = idx;
        SpreadSkin skin = hightIndex == -1 ? new SpreadSkin(list.SelectedItem as SpreadSkin) : new SpreadSkin(list.Items[hightIndex] as SpreadSkin);
        skin.Apply(fpSpread1);
      }
    }

    private void lbSpreadSkins_MouseLeave(object sender, EventArgs e)
    {
      lbSpreadSkins_MouseMove(sender, new MouseEventArgs(System.Windows.Forms.MouseButtons.None, 0, -1, -1, 0));
    }

    private void chkBoxVisibleStatusBar_CheckedChanged(object sender, EventArgs e)
    {
      fpSpread1.StatusBarVisible = chkBoxVisibleStatusBar.Checked;
    }

    private void chkBoxToggleAllElements_CheckedChanged(object sender, EventArgs e)
    {
      EnableAll(chkBoxToggleAllElements.Checked);
    }

    internal void EnableAll(bool enabled)
    {
      foreach (IFpStatusBarElement element in fpSpread1.StatusBar.Elements)
      {
        element.Visible = enabled;
      }
    }

    private void btnAddCustomElement_Click(object sender, EventArgs e)
    {
      MyElement element = new MyElement("Custom Element", "My Element");
      element.Align = ElementAlign.Left;
      fpSpread1.StatusBar.Elements.Add(element);

    }
  }//end class


  public class MyElement : FpBaseStatusBarElement
  {
    public MyElement(string menuText, string text)
    {
      _menuText = menuText;
      _text = text;
    }

    private string _text = "My element";
    public override string Text
    {
      get
      {
        return _text;
      }
    }

    public override int GetPreferedWidth()
    {
      return 300;
    }
    public override void Dopaint(Graphics g, Rectangle r)
    {
      StringFormat format = new StringFormat();
      format.Alignment = StringAlignment.Center;
      format.LineAlignment = StringAlignment.Center;
      g.DrawString(_text, new Font("Arial", 9), Brushes.Red, (RectangleF)r, format);
    }

    private string _menuText = null;
    public override string MenuText
    {
      get
      {
        return String.IsNullOrEmpty(_menuText) ? "NO NAME" : _menuText;
      }
    }
  }

}//end namespace
