using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace SpreadWinDemo.AutoComplete
{
  public partial class AutoComplete : Form
  {
    public AutoComplete ()
    {
      InitializeComponent();
    }

    private void AutoComplete_Load ( object sender, EventArgs e )
    {
      GetSampleData();
      InitData();
    }

    public void GetSampleData ()
    {
      //Add sample data
      //fpData objData = new fpData
      //FpSpread1.DataSource = objData.GetDataSet("Select CompanyName, ContactName, ContactTitle, Country from Customers")

      FpSpread1.DataSource = Features.Databinding.GetNWINDDataComplete("Select CompanyName, ContactName, ContactTitle, Country from Customers");
      //Set the column width to the widest string in the column
      for ( int i = 0; i < FpSpread1.ActiveSheet.Columns.Count; i++ )
        FpSpread1.ActiveSheet.Columns[i].Width = FpSpread1.ActiveSheet.Columns[i].GetPreferredWidth();


      //Always be in edit mode when entering a cell
      FpSpread1.EditModePermanent = true;
      //Highlight the text for replacing when going into cell
      FpSpread1.EditModeReplace = true;

    }
    public void InitData ()
    {
      //Autofill DataSource options
      cbAutofillSource.DataSource = new string[] { "CustomSource and All Column Items Below Cell", "CustomSource Only", "Column Items Below Cell Only" };
      cbAutofillSource.SelectedIndex = 0;

      //AutoComplete Mode options
      cbAutoCompleteMode.DataSource = new string[] { "None", "Suggest", "Append", "SuggestAppend" };
      cbAutoCompleteMode.SelectedIndex = 3;

      //Column options
      cbColumn.DataSource = new string[] { "Company (0)", "Contact (1)", "Title (2)", "Country (3)" };
      cbColumn.SelectedIndex = 0;

    }

    public void CustomAutoComplete ( int column, int acm, int AutoFillAutoCompleteCustomSource )
    {
      //INPUTS:
      //_AutoCompleteMode options
      //   "None", "Suggest", "Append", "SuggestAppend"
      //_AutoFillAutoCompleteCustomSource options
      //   "CustomSource and All Items Below it", "CustomSource Only", "Column Items Below only"

      //Build the custom source suggestion list
      string[] mySuggestionList  = new string[] {"Benjamin", "Bruce", "Cheryl", "Daniel", "Dave",
               "Eric", "Hana", "Harol", "Henry", "Jane", "Jessica", "John", "Julia", "Kathy", "Lance", "Laura",
               "Mark", "Marry", "Meggie", "Mike", "Naomi", "Paul", "Peter", "Roger", "Rose", "Ruth", "Ryan",
               "Shirley", "Tan", "Thomas", "Tony", "Varun", "$1234", "VND1234", "12345 %", "13 %", "02/19/1991",
               "Sunday, August 08, 2007", "Saturday, March 09, 1997", "February", "0901234567", "0953336650"};

      //Bind listbox to show options to user
      lbCustom.DataSource = mySuggestionList;

      //Add the custom source to the collection
      AutoCompleteStringCollection myAutoCompleteStringCollection = new AutoCompleteStringCollection();
      myAutoCompleteStringCollection.AddRange(mySuggestionList);

      //Create the cell type
      FarPoint.Win.Spread.CellType.GeneralCellType myAutoFillCellType = new FarPoint.Win.Spread.CellType.GeneralCellType();

      //AutoCompleteSource sets what source to use for the AutoComplete list items
      //  We//re hard-coding the custom source in this demo. There are many other options available
      myAutoFillCellType.AutoCompleteSource = AutoCompleteSource.CustomSource;

      //AutoCompleteCustomSource specifies a list of items to use when AutoCompleteSource is set to //Custom//
      if ( AutoFillAutoCompleteCustomSource == 2 )  //"Column Items Below only"
        //Set this to nothing to use the items below in this column only - no custom lists!
        myAutoFillCellType.AutoCompleteCustomSource = null;
      else
        //Use the custom list in conjunction with AutoFillAutoCompleteCustomSource setting
        myAutoFillCellType.AutoCompleteCustomSource = myAutoCompleteStringCollection;

      //AutoFillAutoCompleteCustomSource determines whether the custom list is automatically filled with data from the column
      //  true = use the custom source AND the column list below it
      //  false = use custom source list only
      if ( AutoFillAutoCompleteCustomSource == 1 )  //CustomSource Only
        //Use custom source list only
        myAutoFillCellType.AutoFillAutoCompleteCustomSource = false;
      else
        //Use the custom source AND the column list below it
        myAutoFillCellType.AutoFillAutoCompleteCustomSource = true;

      //AutoCompleteMode specifies what type of auto complete to use
      myAutoFillCellType.AutoCompleteMode = (AutoCompleteMode)acm;

      //Assign cell type to column 
      //   could do a cell instead if wanted
      //   FpSpread1.ActiveSheet.Cells(0, 0).CellType = myAutoFillCellType
      FpSpread1.ActiveSheet.Columns[column].CellType = myAutoFillCellType;

    }

    private void cbAutofillSource_SelectedIndexChanged ( object sender, EventArgs e )
    {
      CustomAutoComplete(cbColumn.SelectedIndex, cbAutoCompleteMode.SelectedIndex, cbAutofillSource.SelectedIndex);
    }

    private void cbAutoCompleteMode_SelectedIndexChanged ( object sender, EventArgs e )
    {
      CustomAutoComplete(cbColumn.SelectedIndex, cbAutoCompleteMode.SelectedIndex, cbAutofillSource.SelectedIndex);
    }

    private void cbColumn_SelectedIndexChanged ( object sender, EventArgs e )
    {
      CustomAutoComplete(cbColumn.SelectedIndex, cbAutoCompleteMode.SelectedIndex, cbAutofillSource.SelectedIndex);
    }

  }
}