using System;
using System.Windows.Forms;
using System.Drawing;
using FarPoint.Win;

namespace SpreadWinDemo
{
  /// <summary>
  /// Summary description for customunderline.
  /// </summary>
  public class customunderline : FarPoint.Win.IBorder
  {
    int underlineCount;
    private FarPoint.Win.IBorder baseBorder;
    private FarPoint.Win.Inset m_Inset;

    public customunderline(int underlineCount, FarPoint.Win.IBorder baseBorder)
    {
      if (underlineCount < 1)
      {
        throw new ArgumentException();
      }
      FarPoint.Win.Inset baseinset;
      if (baseBorder == null)
      {
        baseinset = new FarPoint.Win.Inset(0, 0, 0, 0);
      }
      else
      {
        baseinset = baseBorder.Inset;
      }
      this.underlineCount = underlineCount;
      this.baseBorder = baseBorder;
      m_Inset = new FarPoint.Win.Inset(baseinset.Left, baseinset.Top, baseinset.Right, Math.Max(baseinset.Bottom, 2 * underlineCount - 1));
    }
    public FarPoint.Win.Inset Inset
    {
      get
      {
        return m_Inset;
      }
    }
    void FarPoint.Win.IBorder.Paint(System.Drawing.Graphics g, int x, int y, int width, int height)
    {
      if (width > 0 & height > 0)
      {
        int underlineheight = Math.Min(2 * underlineCount - 1, height);
        int underlinewidth = 75 * width / 100;
        int underlinex = x + ((width - underlinewidth) / 2);
        int underliney = y + height - underlineheight;
        g.FillRectangle(System.Drawing.Brushes.White, x, underliney, width, underlineheight);
        if (baseBorder != null)
        {
          baseBorder.Paint(g, x, y, width, height);
        }
        while (underlineheight > 0)
        {
          g.FillRectangle(System.Drawing.Brushes.Red, underlinex, underliney + underlineheight - 1, underlinewidth, 1);
          underlineheight -= 2;
        }
      }
    }
  }   /// <summary>
  /// Summary description for underlineborder.
  /// </summary>
  public class underlineborder : FarPoint.Win.IBorder
  {

    int underlineCount;
    private FarPoint.Win.Inset m_Inset;
    private FarPoint.Win.IBorder baseBorder;

    public FarPoint.Win.Inset Inset
    {
      get
      {
        return m_Inset;
      }
    }

    public underlineborder(int underlineCount, FarPoint.Win.IBorder baseBorder)
    {
      if (underlineCount < 1)
      {
        throw new ArgumentException();
      }
      FarPoint.Win.Inset baseinset;
      if (baseBorder == null)
      {
        baseinset = new FarPoint.Win.Inset(0, 0, 0, 0);
      }
      else
      {
        baseinset = baseBorder.Inset;
      }

      this.underlineCount = underlineCount;
      this.baseBorder = baseBorder;
      m_Inset = new FarPoint.Win.Inset(baseinset.Left, baseinset.Top, baseinset.Right, Math.Max(baseinset.Bottom, 2 * underlineCount - 1));

    }
    void FarPoint.Win.IBorder.Paint(System.Drawing.Graphics g, int x, int y, int width, int height)
    {
      if (width > 0 & height > 0)
      {
        int underlineheight = Math.Min(2 * underlineCount - 1, height);
        int underlinewidth = width;
        int underlinex = x + ((width - underlinewidth) / 2);
        int underliney = y + height - underlineheight;
        g.FillRectangle(System.Drawing.Brushes.White, x, underliney, width, underlineheight);
        if (baseBorder != null)
        {
          baseBorder.Paint(g, x, y, width, height);
        }
        while (underlineheight > 0)
        {
          g.FillRectangle(System.Drawing.Brushes.Red, underlinex, underliney + underlineheight - 1, underlinewidth, 1);
          underlineheight -= 2;
        }

      }
    }
  }
  public class AntiqueBorder : FarPoint.Win.IBorder
  {
    private FarPoint.Win.Inset m_Inset;
    private Color m_clr = Color.Black;
    private Color matteColor = Color.White;

    public AntiqueBorder(Color c, bool matte, Color matteColor)
    {
      m_clr = c;
      this.matteColor = matteColor;
      int thick = matte ? 20 : 8;
      m_Inset = new FarPoint.Win.Inset(thick, thick, thick, thick);
    }

    //Inset determines how the contents of the cell are offset from the edge. To create the matte effect we assign a value larger than the actual size of the border.
    public FarPoint.Win.Inset Inset
    {
      get
      {
        return m_Inset;
      }
    }

    public Color Color
    {
      get
      {
        return m_clr;
      }
    }
    public Color MatteColor
    {
      get
      {
        return matteColor;
      }
    }
    void FarPoint.Win.IBorder.Paint(System.Drawing.Graphics g, int x, int y, int width, int height) // FarPoint.Win.IBorder.Paint
    {
      if (width > 0 & height > 0)
      {
        Pen mattePen = new Pen(matteColor);
        int matteThickness = m_Inset.Bottom;

        for (int i = 0; i < matteThickness && i < width; i++)
          g.DrawLine(mattePen, x + i, y, x + i, y + height - 1);
        for (int i = 0; i < matteThickness && i < height; i++)
          g.DrawLine(mattePen, x, y + i, x + width - 1, y + i);
        for (int i = 0; i < matteThickness && i < width; i++)
          g.DrawLine(mattePen, x + width - 1 - i, y, x + width - 1 - i, y + height - 1);
        for (int i = 0; i < matteThickness && i < height; i++)
          g.DrawLine(mattePen, x, y + height - 1 - i, x + width - 1, y + height - 1 - i);
        mattePen.Dispose();

        SolidBrush b = new SolidBrush(m_clr);
        SolidBrush mb = new SolidBrush(matteColor);
        int thinWidth = System.Math.Min(3, width);
        int mediumWidth = System.Math.Min(5, width);
        int thickWidth = System.Math.Min(7, width);
        int thinHeight = System.Math.Min(3, height);
        int mediumHeight = System.Math.Min(5, height);
        int thickHeight = System.Math.Min(7, height);
        g.FillRectangle(mb, x, y, mediumWidth, height);
        g.FillRectangle(mb, x, y, width, mediumHeight);
        g.FillRectangle(mb, x + width - mediumWidth, y, mediumWidth, height);
        g.FillRectangle(mb, x, y + height - mediumHeight, width, mediumHeight);
        g.FillRectangle(b, x, y, thinWidth, height);
        g.FillRectangle(b, x, y, width, thinHeight);
        g.FillRectangle(b, x + width - thinWidth, y, thinWidth, height);
        g.FillRectangle(b, x, y + height - thinHeight, width, thinHeight);
        g.FillRectangle(b, x, y, thickWidth, mediumHeight);
        g.FillRectangle(b, x, y, mediumWidth, thickHeight);
        g.FillRectangle(b, x + width - thickWidth, y, thickWidth, mediumHeight);
        g.FillRectangle(b, x + width - mediumWidth, y, mediumWidth, thickHeight);
        g.FillRectangle(b, x + width - thickWidth, y + height - mediumHeight, thickWidth, mediumHeight);
        g.FillRectangle(b, x + width - mediumWidth, y + height - thickHeight, mediumWidth, thickHeight);
        g.FillRectangle(b, x, y + height - mediumHeight, thickWidth, mediumHeight);
        g.FillRectangle(b, x, y + height - thickHeight, mediumWidth, thickHeight);
        b.Dispose();
        mb.Dispose();
      }
    }

  }

  public class LineBorderNew : IBorder
  {

    private Color color;
    private int thickness = 7;
    private Inset inset;
    private Color matteColor;

    public LineBorderNew(Color color, bool matte, Color matteColor)
    {
      this.color = color;
      this.matteColor = matteColor;
      this.inset = new Inset(matte ? 20 : 0);
    }


    public Color Color
    {
      get
      {
        return color;
      }
    }

    public Color MatteColor
    {
      get
      {
        return matteColor;
      }
    }

    //Inset determines how the contents of the cell are offset from the edge. To create the matte effect we assign a value larger than the actual size of the border.
    public Inset Inset
    {
      get
      {
        return inset;
      }
    }

    public int Thickness
    {
      get
      {
        return thickness;
      }
    }

    public virtual void Paint(Graphics g, int x, int y, int width, int height)
    {
      InnerPaint(g, x, y, width, height);
    }


    private void InnerPaint(Graphics g, float x, float y, float width, float height)
    {

      Pen mattePen = new Pen(matteColor);
      int matteThickness = inset.Bottom;

      for (int i = 0; i < matteThickness && i < width; i++)
        g.DrawLine(mattePen, x + i, y, x + i, y + height - 1);
      for (int i = 0; i < matteThickness && i < height; i++)
        g.DrawLine(mattePen, x, y + i, x + width - 1, y + i);
      for (int i = 0; i < matteThickness && i < width; i++)
        g.DrawLine(mattePen, x + width - 1 - i, y, x + width - 1 - i, y + height - 1);
      for (int i = 0; i < matteThickness && i < height; i++)
        g.DrawLine(mattePen, x, y + height - 1 - i, x + width - 1, y + height - 1 - i);
      mattePen.Dispose();


      Pen pen = new Pen(color);
      for (int i = 0; i < thickness && i < width; i++)
        g.DrawLine(pen, x + i, y, x + i, y + height - 1);
      for (int i = 0; i < thickness && i < height; i++)
        g.DrawLine(pen, x, y + i, x + width - 1, y + i);
      for (int i = 0; i < thickness && i < width; i++)
        g.DrawLine(pen, x + width - 1 - i, y, x + width - 1 - i, y + height - 1);
      for (int i = 0; i < thickness && i < height; i++)
        g.DrawLine(pen, x, y + height - 1 - i, x + width - 1, y + height - 1 - i);
      pen.Dispose();
    }

  }

  /// <summary>
  /// Summary description for Borders.
  /// </summary>
  public class GlossyBorder : FarPoint.Win.IBorder
  {
    private Color m_clr = Color.Black;
    private FarPoint.Win.Inset m_Inset;
    private Color matteColor = Color.White;

    public GlossyBorder(Color c, bool matte, Color matteColor)
    {
      m_clr = c;
      int thick = matte ? 20 : 0;
      this.matteColor = matteColor;
      m_Inset = new Inset(thick, thick, thick, thick);
    }

    //Inset determines how the contents of the cell are offset from the edge. To create the matte effect we assign a value larger than the actual size of the border.
    public FarPoint.Win.Inset Inset
    {
      get { return m_Inset; }
    }
    public Color Color
    {
      get
      {
        return m_clr;
      }
    }
    public Color MatteColor
    {
      get
      {
        return matteColor;
      }
    }

    void FarPoint.Win.IBorder.Paint(System.Drawing.Graphics g, int x, int y, int width, int height)
    {
      if (width > 0 & height > 0)
      {
        Pen mattePen = new Pen(matteColor);
        int matteThickness = m_Inset.Bottom;

        for (int i = 0; i < matteThickness && i < width; i++)
          g.DrawLine(mattePen, x + i, y, x + i, y + height - 1);
        for (int i = 0; i < matteThickness && i < height; i++)
          g.DrawLine(mattePen, x, y + i, x + width - 1, y + i);
        for (int i = 0; i < matteThickness && i < width; i++)
          g.DrawLine(mattePen, x + width - 1 - i, y, x + width - 1 - i, y + height - 1);
        for (int i = 0; i < matteThickness && i < height; i++)
          g.DrawLine(mattePen, x, y + height - 1 - i, x + width - 1, y + height - 1 - i);
        mattePen.Dispose();

        SolidBrush b1 = new SolidBrush(m_clr);
        SolidBrush b2 = new SolidBrush(ControlPaint.Light(m_clr));
        SolidBrush b3 = new SolidBrush(ControlPaint.LightLight(m_clr));
        SolidBrush b4 = new SolidBrush(ControlPaint.Dark(m_clr));

        g.FillRectangle(b4, x, y, width, 1);
        g.FillRectangle(b1, x + 1, y + 1, width - 2, 1);
        g.FillRectangle(b2, x + 2, y + 2, width - 4, 1);
        g.FillRectangle(b3, x + 3, y + 3, width - 6, 1);
        g.FillRectangle(b2, x + 4, y + 4, width - 8, 1);
        g.FillRectangle(b1, x + 5, y + 5, width - 10, 1);
        g.FillRectangle(b4, x + 6, y + 6, width - 12, 1);

        g.FillRectangle(b4, x, y + height - 1, width, 1);
        g.FillRectangle(b1, x + 1, y + height - 2, width - 2, 1);
        g.FillRectangle(b2, x + 2, y + height - 3, width - 4, 1);
        g.FillRectangle(b3, x + 3, y + height - 4, width - 6, 1);
        g.FillRectangle(b2, x + 4, y + height - 5, width - 8, 1);
        g.FillRectangle(b1, x + 5, y + height - 6, width - 10, 1);
        g.FillRectangle(b4, x + 6, y + height - 7, width - 12, 1);

        g.FillRectangle(b4, x, y, 1, height);
        g.FillRectangle(b1, x + 1, y + 1, 1, height - 2);
        g.FillRectangle(b2, x + 2, y + 2, 1, height - 4);
        g.FillRectangle(b3, x + 3, y + 3, 1, height - 6);
        g.FillRectangle(b2, x + 4, y + 4, 1, height - 8);
        g.FillRectangle(b1, x + 5, y + 5, 1, height - 10);
        g.FillRectangle(b4, x + 6, y + 6, 1, height - 12);

        g.FillRectangle(b4, x + width - 1, y, 1, height);
        g.FillRectangle(b1, x + width - 2, y + 1, 1, height - 2);
        g.FillRectangle(b2, x + width - 3, y + 2, 1, height - 4);
        g.FillRectangle(b3, x + width - 4, y + 3, 1, height - 6);
        g.FillRectangle(b2, x + width - 5, y + 4, 1, height - 8);
        g.FillRectangle(b1, x + width - 6, y + 5, 1, height - 10);
        g.FillRectangle(b4, x + width - 7, y + 6, 1, height - 12);


      }
    }
  }
}
